/* Include some header files */
#include "aprecv-structs.h"
#include "aprecv-globals.h"

/* Prototypes */

/* parseoptions.c */
int parseoptions                (int, char **);          

/* usage.c */
int usage                       (char *, char *);         
int showversion 		(void);			 

/* error.c */
void clean_exit			(int);                    

/* statistic.c */	
void print_statistic		(void);
void cap_stats			(void);			  

/* read_iptrace.c */
void read_iptrace	        (int);

/* read_snoop.c */
void read_snoop			(int);

/* receive_pcap.c */
int start_sniffing_pcap		(char *, int, int, char *, int, int);	
int recv_packet_pcap		(unsigned char *, struct pcap_pkthdr*, unsigned char *);

/* receive_apsrlib.c */
int start_sniffing_apsr         (char *, int, int, int);
int recv_packet_apsr 		(struct sockaddr *, unsigned char *, unsigned int);

#ifdef HAVE_APSRLIB_BLUETOOTH_SUPPORT
/* receive_bluetooth.c */
int start_sniffing_bluetooth    (char *, int);
int recv_packet_bluetooth       (unsigned char *, unsigned int);
int print_hci_packet		(struct aprecv_frame_header*, unsigned char *, int);
#endif

/* print-ethernet.c */
int print_ethernet_packet	(struct aprecv_frame_header*, unsigned char *, int);
void print_ethernet_proto       (unsigned short);
void switchto_ethernet_proto    (struct aprecv_frame_header*, unsigned char *, int, unsigned short);

/* print-chdlc.c */
int print_chdlc_packet          (struct aprecv_frame_header*, unsigned char *, int);

/* print-sunatm.c */
int print_sunatm_packet         (struct aprecv_frame_header*, unsigned char *, int);

/* print-llc.c */
int print_llc_packet            (struct aprecv_frame_header*, unsigned char *, int, 
		                 unsigned short);
void print_snap_oui             (unsigned int);

/* print-fddi.c */
int print_fddi_packet     	(struct aprecv_frame_header*, unsigned char *, int);

/* print-bofl.c */
int print_bofl_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-mpls.c */
int print_mpls_packet           (unsigned char *, int);

/* print-gsmp.c */
int print_gsmp_packet           (unsigned char *, int);

/* print-mtp.c */
int print_mtp_packet 		(unsigned char *, int);

/* print-pgm.c */
int print_pgm_packet		(unsigned char *, int);

/* print-sdrp.c */
int print_sdrp_packet           (unsigned char *, int);

/* print-rsvp.c */
int print_rsvp_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-token.c */
int print_token_packet		(struct aprecv_frame_header*, unsigned char *, int);

/* print-atm_rfc1483.c */
int print_atm_rfc1483_packet    (struct aprecv_frame_header*, unsigned char *, int);

/* print-atm_clip.c */
int print_atm_clip_packet    	(struct aprecv_frame_header*, unsigned char *, int);

/* print-ipv4.c */
int print_ipv4_packet		(struct aprecv_frame_header*, unsigned char *, int);    
void print_ipv4_proto		(unsigned short);
void switchto_ipv4_proto	(struct aprecv_frame_header*, unsigned char *, int, unsigned short);

/* print-ipv6.c */
int print_ipv6_packet		(struct aprecv_frame_header*, unsigned char *, int);
int print_ipv6_rt_packet        (struct aprecv_frame_header*, unsigned char *, int);
int print_ipv6_hh_packet	(struct aprecv_frame_header*, unsigned char *, int);
void print_ipv6_proto           (unsigned char);
void switchto_ipv6_proto        (struct aprecv_frame_header*, unsigned char *, int, unsigned char);

/* print-ipv7.c */
int print_ipv7_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-ipah.c */
int print_ipah_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-pppoe.c */
int print_pppoe_packet		(struct aprecv_frame_header*, unsigned char *, int);		 
int print_pppoe_tag		(unsigned char *);

/* print-ppp.c */
int print_ppp_packet		(struct aprecv_frame_header*, unsigned char *, int);
int print_ppp_lcp		(unsigned char *, int);
int print_ppp_ipcp		(unsigned char *, int);
int print_ppp_ipv6cp            (unsigned char *, int);
int print_ppp_ipxcp             (unsigned char *, int);
int print_ppp_atcp              (unsigned char *, int);
int print_ppp_ecp               (unsigned char *, int);
int print_ppp_ccp               (unsigned char *, int);
int print_ppp_pap		(unsigned char *, int);
int print_ppp_chap              (unsigned char *, int);
int print_ppp_eap               (unsigned char *, int);
int print_ppp_bcp               (unsigned char *, int);
int print_ppp_lqr               (unsigned char *, int);
int print_ppp_decnetcp		(unsigned char *, int);
int print_ppp_osinlcp		(unsigned char *, int);
int print_ppp_snacp		(unsigned char *, int);

/* print-arp.c */
int print_arp_packet		(unsigned char *, int);

/* print-udp.c */
int print_udp_packet		(struct aprecv_frame_header*, unsigned char *, int, int);

/* print-tcp.c */
int print_tcp_packet		(struct aprecv_frame_header*, unsigned char *, int, int);
void print_tcp_streams		(void);
void rev_print_list		(struct tcp_stream *);

/* print-icmpv4.c */ 	
int print_icmpv4_packet		(struct aprecv_frame_header*, unsigned char *, int);

/* print-icmpv6.c */
int print_icmpv6_packet         (unsigned char *, int);

/* print-igmp.c */
int print_igmp_packet		(struct aprecv_frame_header*, unsigned char *, int);
void print_igmp_type		(unsigned char);

/* print-rip.c */
int print_rip_packet		(unsigned char *, int);
int print_ripng_packet          (unsigned char *, int);
int print_ipxrip_packet         (unsigned char *, int);

/* print-ipx.c */
int print_ipx_packet            (unsigned char *, int);

/* print-spx.c */
int print_spx_packet            (unsigned char *, int);

/* print-netblk.c */
int print_netblk_packet         (unsigned char *, int);

/* print-tmux.c */
int print_tmux_mini_packet      (unsigned char *, int);

/* print-ospf.c */
int print_ospf_packet           (unsigned char *, int);

/* print-cbt.c */ 
int print_cbt_packet 		(struct aprecv_frame_header*, unsigned char *,
		                 int);

/* print-etherip.c */
int print_etherip_packet        (struct aprecv_frame_header*, unsigned char *, int);

/* print-bgp.c */
int print_bgp_packet            (unsigned char *, int);

/* print-msdp.c */
int print_msdp_packet           (unsigned char *, int);

/* print-egp.c */
int print_egp_packet            (struct aprecv_frame_header*, unsigned char *, int);

/* print-sctp.c */
int print_sctp_packet           (unsigned char *, int);

/* print-ipcomp.c */
int print_ipcomp_packet         (unsigned char *, int);

/* print-narp.c */
int print_narp_packet           (unsigned char *, int);

/* print-ggp.c */
int print_ggp_packet            (unsigned char *, int);

/* print-rdp.c */
int print_rdp_packet            (unsigned char *, int);

/* print-igrp.c */
int print_igrp_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-eigrp.c */
int print_eigrp_packet          (struct aprecv_frame_header*, unsigned char *, int);

/* print-irtp.c */
int print_irtp_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-raw.c */
int print_raw_in_hex		(unsigned char *, int);   
int print_raw_in_text		(unsigned char *, int);

/* print-pim.c */
int print_pim_packet            (unsigned char *, int);

/* print-ipesp.c */
int print_ipesp_packet          (unsigned char *, int);

/* print-802_11.c */
int print_802_11_packet		(struct aprecv_frame_header*, unsigned char *, int);

/* print-prism.c */
int print_prism_mon_packet      (struct aprecv_frame_header*, unsigned char *, int);

/* print-sll.c */
int print_sll_packet            (struct aprecv_frame_header*, unsigned char *, int);

/* print-gre.c */
int print_gre_packet            (struct aprecv_frame_header*, unsigned char *, int);

/* print-ifmp.c */
int print_ifmp_packet           (unsigned char *, int);

/* print-cdp.c */
int print_cdp_packet            (struct aprecv_frame_header*, unsigned char *, int, unsigned short);

/* print-cgmp.c */
int print_cgmp_packet           (unsigned char *, int);

/* print-vtp.c */
int print_vtp_packet           (unsigned char *, int);

/* print-null.c */
int print_null_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-slip.c */
int print_slip_packet           (struct aprecv_frame_header*, unsigned char *, int);

/* print-vrrp.c */
int print_vrrp_packet           (unsigned char *, int);

/* print-mopv3.c */
int print_mopv3_dumpload_packet (unsigned char *, int);

/* daemon.c */
int daemon_mode			(void);			  

/* resolve.c */
int resolve_ip			(char *, int);		  
int resolve_mac			(unsigned char *);	  

#ifdef HAVE_APSRLIB_MODULE_SUPPORT
/* load_module.c */
int load_module			(char *);
void load_modules_in_path	(char *);
void find_module		(unsigned char *, int , unsigned short, unsigned short, unsigned short, unsigned short);
void close_modules		();
struct apsr_aprecv_module_info (*get_module_info)	(void);
#endif

/* apsr-communication.c */
int open_server_connection	(char *, unsigned short);
int close_server_connection	(void);
int send_server_got_packet	(const unsigned char *, unsigned short);

