#include <netinet/in.h>

#ifdef HAVE_APSRLIB_MODULE_SUPPORT
typedef int (*apsr_module_init)(unsigned char *, int, unsigned short, unsigned short, unsigned short);

struct apsr_modules { 
        struct apsr_modules		*next;
        void                    	*dl_handle;
        apsr_module_init       		init;
        struct apsr_aprecv_module_info	info;
};
#endif

struct aprecv_frame_header {
  unsigned int caplen;
  unsigned int len;
};

/* Dummy unsigned char to in_addr struct */
struct dummy_ip {
  struct in_addr ip_address;	/* IPv4 Address */
};

/* TCP Stream struct */
struct tcp_stream {
  struct tcp_stream *next;
  /* IPv4 header */
  unsigned int ipvhl:8;         /* Version + Header length      */
  unsigned int type_service:8;  /* TOS(Type of Service) field   */
  unsigned short packet_len;    /* Header+Payload length        */
  unsigned short ident;         /* Identification field         */
  unsigned short fragment;      /* Fragment Offset field        */
  unsigned int time_live:8;     /* TTL(Time to Live) field      */
  unsigned int protocol:8;      /* Protocol field               */
  unsigned short ip_sum;        /* Checksum field               */
  struct in_addr src_ip;        /* Source IP                    */
  struct in_addr dst_ip;        /* Destination IP               */
  /* TCP header */
  unsigned short src_port;      /* Source Port                  */
  unsigned short dst_port;      /* Destination Port             */
  unsigned int seq_num;         /* Sequence Number              */
  unsigned int ack_num;         /* Acknowledgement Number       */
  unsigned int reserved:4,      /* (unused) */
               data_offset:4,   /* data offset */
               flags:8,         /* Flags                        */
               window:16;       /* Window                       */
  unsigned short tcp_sum;       /* Checksum                     */
  unsigned short urgent;        /* Urgent Pointer               */
  /* Payload */
  unsigned short payload_length; 
  unsigned char payload[65495]; /* 65535 - 20 bytes std. IPv4 header and 
  				 * - 20 bytes std TCP header */
};

/* IPv4 options */
struct ipv4_options {
  union {
	  struct {
            unsigned int option:8;	/* Option */
	    unsigned int length:8;	/* Length */
          } dummy;	
  } ext;	 
};

/* TCP options */
struct tcp_options {
  union {
	struct {	/* Maximum Segment Size */
	  unsigned int kind_option:8;   /* Option */
	  unsigned int length:8;        /* Length */
	  unsigned short mss;           /* Maximum Segment Size */
        } mss;
        struct {	/* Timestamp */
          unsigned int kind_option:8;   /* Option */
          unsigned int length:8;        /* Length */
          unsigned long ts_value;       /* TSVal */ 
	  unsigned long ts_echo;	/* TSecr */
        } time;
        struct {        /* Window Scale */
          unsigned int kind_option:8;   /* Option */
          unsigned int length:8;        /* Length */
          unsigned int ws:8;            /* Window Scale Factor */
        } ws;
        struct {        /* Connection Count */
          unsigned int kind_option:8;   /* Option */
          unsigned int length:8;        /* Length */ 
          unsigned long cc;             /* Connection Count */
        } cc;
        struct {	/* Alternate Checksum */
          unsigned int kind_option:8;	/* Option */
          unsigned int length:8;	/* Length */
          unsigned int type_sum:8;	/* Type of Checksum */
        } alter_chksum;
  } ext;
};

/* Ethereal struct */
struct iptrace_v2_pkt_hdr {
	unsigned int    pkt_length;     /* packet length + 32 */        
	unsigned int    tv_sec0;        /* time stamp seconds, since 1970 */
	unsigned int    unknown1;       /* ?? */
	char            if_name[4];     /* null-terminated */
	char            if_desc[12];    /* interface description. */
	unsigned int    if_type:8;      /* BSD net/if_types.h */
	unsigned int    tx_flag:8;      /* 0=receive, 1=transmit */
        unsigned short  unknown2;
        unsigned int    tv_sec;         /* time stamp, seconds since the 1970 */
        unsigned int    tv_nsec;        /* nanoseconds since that second */
};

/* Snoop header of trace file */
struct snoop_file_hdr {
	unsigned char identification[8];
	unsigned int version;
	unsigned int datalink_type;
};

/* Snoop header for every packet */
struct snoop_pkt_hdr {
	unsigned int original_length;
	unsigned int included_length;
	unsigned int packet_record_length;
	unsigned int cumulative_drops;
	unsigned int timestamp_seconds;
	unsigned int timestamp_microsec;
};

