#include "../include/aprecv-general.h"

int daemon_mode (void) {
  FILE *pidfile_handle=NULL;
  /* Solaris doesn't know pid_t :-) */
  #ifndef pid_t	
  typedef int pid_t; 
  #endif

  pid_t pid;
	
  if((pid = fork()) < 0) {
	  #ifdef APRECV_DEBUG
	  apsr_stream_output((FILE *) stderr, "\nError: fork failed(%s).\n", strerror(errno));
	  #endif 
	  return(RET_FAILURE);
  }
  else if (pid != 0) {
	  #ifdef APRECV_DEBUG
	  apsr_stream_output((FILE *) stderr, "\nStarted daemon with pid %d.\n", pid);
	  #endif

   	  exit(EXIT_SUCCESS);	/* Kiss goodbye parent */
  }
  
  if(setsid() < 0) {
	  #ifdef APRECV_DEBUG
	  apsr_stream_output((FILE *) stderr, "\nError: setsid failed(no permissions).\n");
	  #endif
  }		  
  
  if(aprecv_log_file) {
    if(!(freopen("/dev/null", "r", stdin))) {
	    apsr_stream_output((FILE *) stderr, "Error: Can't open file /dev/null(daemon).\n");
	    #ifdef APRECV_DEBUG
	    apsr_stream_output((FILE *) stderr, "freopen %s.\n", strerror(errno));
	    #endif
    }
    if(!(freopen(aprecv_log_file, "a", stdout))) {
	    apsr_stream_output((FILE *) stderr, "Error: Can't open logfile(daemon) %s.\n", aprecv_log_file);
            #ifdef APRECV_DEBUG
    	    apsr_stream_output((FILE *) stderr, "freopen %s.\n", strerror(errno));
 	    #endif
    }
    if(!(freopen(aprecv_log_file, "a", stderr))) {
	    apsr_stream_output((FILE *) stderr, "Error: Can't open logfile(daemon) %s.\n", aprecv_log_file);
            #ifdef APRECV_DEBUG
	    apsr_stream_output((FILE *) stderr, "freopen %s.\n", strerror(errno)); 
            #endif
    }
  }
  else {
    /* I think we don't need it if no one will read it, maybe syslog? */
    if(fclose(stdin) < 0) {
	    apsr_stream_output((FILE *) stderr, "Error: Can't close stdin(daemon) filehandle.\n");
            #ifdef APRECV_DEBUG
	    apsr_stream_output((FILE *) stderr, "fclose %s.\n", strerror(errno));
	    #endif
    }
    if(fclose(stdout) < 0) {
	    apsr_stream_output((FILE *) stderr, "Error: Can't close stdout(daemon) filehandle.\n");
	    #ifdef APRECV_DEBUG
	    apsr_stream_output((FILE *) stderr, "fclose %s.\n", strerror(errno));
	    #endif
    }
    if(fclose(stderr) < 0) {
	    apsr_stream_output((FILE *) stderr, "Error: Can't close stderr(daemon) filehandle.\n");
            #ifdef APRECV_DEBUG
	    apsr_stream_output((FILE *) stderr, "fclose %s.\n", strerror(errno));
            #endif
    }
  }

  if((pidfile_handle = fopen("/var/run/aprecv.pid", "wb")) == NULL) {
	  apsr_stream_output((FILE *) stderr, "Error: Can't open pid file.\n");
	  #ifdef APRECV_DEBUG
	  apsr_stream_output((FILE *) stderr, "Error code: %s.\n",
			  strerror(errno));
          #endif	    
  }		

  if(pidfile_handle) {
	  apsr_stream_output((FILE *) pidfile_handle, "%d\n", getpid());
          if(fclose(pidfile_handle) < 0) {
		  apsr_stream_output((FILE *) stderr, "Error: Can't close pid file.\n");
		  #ifdef APRECV_DEBUG
		  apsr_stream_output((FILE *) stderr, "Error code: %s.\n", 
				  strerror(errno));
		  #endif
          }
  }					
  
  return(RET_SUCCESS);	
}
