#include "../include/aprecv-general.h"

int print_ethernet_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  struct ethernet_pkt_header *ethernet_header;
  struct vlan_pkt_header *vlan_header=0;
  unsigned short ethernet_type=0, ethernet_vlan_type=0; 
  unsigned short next_protocol=0, vlan=0;

  ethernet_header = (struct ethernet_pkt_header *)(packet+opt_size);
  ethernet_type = apsr_ntohs(ethernet_header->type);

  apsr_output("\n- Ethernet Header -");

  if(header->caplen < APSR_ETH) {
	  apsr_output("\nError: Captured data length(%d bytes) < Ethernet packet header length.\n", (int) header->caplen);
	  if(statistic)
		  stats.tr.ethernetII++;
	  goto end_ethernet;
  }

  if(expand_ethernet || expand_all) {
    apsr_output("\n  MAC Dst: %s", ether2str(ethernet_header->destination));
    if(verbose)
         resolve_mac(ethernet_header->destination);
	 
    apsr_output("\n  MAC Src: %s", ether2str(ethernet_header->source));
    if(verbose)
 	 resolve_mac(ethernet_header->source);
  }
    
  if(ethernet_type == APSR_ETHERTYPE_VLAN) {       /* VLAN 802.1p/1q frames */
        opt_size += 4;
        vlan_header = (struct vlan_pkt_header *)(packet+APSR_ETH);
        if(expand_ethernet || expand_all) {
          apsr_output("\n  TPID: 0x%04x", ethernet_type);
	  vlan = apsr_ntohs(vlan_header->vlan);
	  apsr_output("\n  Priority: %d, TR/CFI: %d", vlan >> 13, 
   	  	      vlan & 0x1000);
	  apsr_output("\n  VLAN-ID: %d", 
		vlan & 0x0fff);
        }
	
        ethernet_vlan_type = apsr_ntohs(vlan_header->type);
        if(ethernet_vlan_type <= 1500) 
	  print_llc_packet(header, packet, opt_size, ethernet_vlan_type);
        else {
          if(expand_ethernet || expand_all) {
   	    apsr_output("\n  Type: 0x%04x", ethernet_vlan_type);
	    if(verbose) 
		  print_ethernet_proto(ethernet_vlan_type);
	  }
          next_protocol = ethernet_vlan_type;
        }
	
	if(statistic)
		stats.c.vlan++;
  }
  else if(ethernet_type > 1500) {	/* Ethernet II or 802.x/SNAP frames */
        next_protocol = ethernet_type;
        if(expand_ethernet || expand_all) {
	  apsr_output("\n  Type(Ethernet II): 0x%04x", ethernet_type);
  
          if(verbose) 
		print_ethernet_proto(ethernet_type);
	}  
	  
        if(statistic)
		stats.c.ethernetII++;  
  }	
  else if (ethernet_type <= 1500) {	/* 802.3, for now it a length field */
        if(ethernet_header->ext.snap.dsap == 0xff && ethernet_header->ext.snap.ssap == 0xff) {
                opt_size += 2;
                if(expand_ethernet || expand_all) {
			apsr_output("\n  Length(802.3raw): %d",(int) ethernet_type);
		}

                if(statistic)
                  stats.c.ethernet8023raw++;

                print_ipx_packet(packet, opt_size);
	}
	else 
          print_llc_packet(header, packet, opt_size, ethernet_type);
  }
	    
  if(next_protocol != 0) {
	switchto_ethernet_proto(header, packet, opt_size, next_protocol);
  }

end_ethernet:
  return(RET_SUCCESS);
}

void switchto_ethernet_proto(struct aprecv_frame_header *header, unsigned char *packet, int opt_size, unsigned short protocol) {
  switch(protocol) {
	  case APSR_ETHERTYPE_ARP:
	    print_arp_packet(packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_REVARP:
	    print_arp_packet(packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_IP:
	    print_ipv4_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_IP6:
	    print_ipv6_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_PPP:
	    print_ppp_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_PPPOE_D:
	    print_pppoe_packet(header, packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_PPPOE_S:
	    print_pppoe_packet(header, packet, opt_size);
	    break;
          case APSR_ETHERTYPE_IPX:
            print_ipx_packet(packet, opt_size);
            break;
          case APSR_ETHERTYPE_DEC_MOP_DUMP:
	    print_mopv3_dumpload_packet(packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_MPLS_UNI:
	  case APSR_ETHERTYPE_MPLS_MULTI:
	    print_mpls_packet(packet, opt_size);
	    break;
	  case APSR_ETHERTYPE_WELLFLEET_BOFL:
	    print_bofl_packet(header, packet, opt_size);
	    break;
          default:
            apsr_output("\n");
            break;
  }
}

void print_ethernet_proto(unsigned short protocol) {
  switch(protocol) {
	  case APSR_ETHERTYPE_ARP:
	    apsr_output(" (ARP)");
	    break;
	  case APSR_ETHERTYPE_REVARP:
	    apsr_output(" (Reverse ARP)");
	    break;
	  case APSR_ETHERTYPE_IP:
	    apsr_output(" (IPv4)");
	    break;
	  case APSR_ETHERTYPE_IP6:
	    apsr_output(" (IPv6)");
	    break;
	  case APSR_ETHERTYPE_PPP:
	    apsr_output(" (PPP)");
	    break;
	  case APSR_ETHERTYPE_LOOPBACK:
	    apsr_output(" (Ethernet Loopback)");
	    break;
	  case APSR_ETHERTYPE_PPPOE_D:
	    apsr_output(" (PPPoE - Discovery Stage)");
	    break;
	  case APSR_ETHERTYPE_PPPOE_S:
	    apsr_output(" (PPPoE - Session Stage)");
	    break;
          case APSR_ETHERTYPE_IPX:
            apsr_output(" (IPX)");
            break;
	  case APSR_ETHERTYPE_DEC_MOP_DUMP:
	    apsr_output(" (DEC Dump/Load)");
	    break;
	  case APSR_ETHERTYPE_PUP:
	    apsr_output(" (PUP)");
	    break;
	  case APSR_ETHERTYPE_VLAN:
	    apsr_output(" (VLAN)");
	    break;
	  case APSR_ETHERTYPE_PUB_TRANS1:
	    apsr_output(" (PUB_TRANS1)");
	    break;
          case APSR_ETHERTYPE_NIXDORF:
	    apsr_output(" (NIXDORF)");
	    break;
	  case APSR_ETHERTYPE_NSIDP:
	    apsr_output(" (NSIDP)");
	    break;
	  case APSR_ETHERTYPE_DLOG1:
	    apsr_output(" (DLOG1)");
	    break;
	  case APSR_ETHERTYPE_DLOG2:
	    apsr_output(" (DLOG2)");
	    break;
	  case APSR_ETHERTYPE_X75:
	    apsr_output(" (X75)");
	    break;
	  case APSR_ETHERTYPE_NBS:
	    apsr_output(" (NBS)");
	    break;
	  case APSR_ETHERTYPE_ECMA:
	    apsr_output(" (ECMA)");
	    break;
	  case APSR_ETHERTYPE_CHAOSNET:
	    apsr_output(" (CHAOSNET)");
	    break;
	  case APSR_ETHERTYPE_X25_LEVEL3:
	    apsr_output(" (X25_LEVEL3)");
	    break;
	  case APSR_ETHERTYPE_XNS1:
	    apsr_output(" (XNS1)");
	    break;
	  case APSR_ETHERTYPE_FRAME_RELAY_ARP:
	    apsr_output(" (FRAME_RELAY_ARP)");
	    break;
	  case APSR_ETHERTYPE_SYMBOLICS_PRIVATE:
	    apsr_output(" (SYMBOLICS_PRIVATE)");
	    break;
	  case APSR_ETHERTYPE_XYPLEX1:
	    apsr_output(" (XYPLEX1)");
	    break;
	  case APSR_ETHERTYPE_XYPLEX2:
	    apsr_output(" (XYPLEX2)");
	    break;
	  case APSR_ETHERTYPE_XYPLEX3:
	    apsr_output(" (XYPLEX3)");
	    break;
	  case APSR_ETHERTYPE_UNGERMANN_BASS:
	    apsr_output(" (UNGERMANN_BASS)");
	    break;
	  case APSR_ETHERTYPE_IEEE802_3_PUP:
	    apsr_output(" (IEEE802_3_PUP)");
	    break;
	  case APSR_ETHERTYPE_VINES:
	    apsr_output(" (VINES)");
	    break;
          case APSR_ETHERTYPE_VINES_LO:
	    apsr_output(" (VINES_LO)");
	    break;
	  case APSR_ETHERTYPE_VINES_ECHO:
	    apsr_output(" (VINES_ECHO)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER1:
	    apsr_output(" (BERKLEY_TRAILER1)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER2:
	    apsr_output(" (BERKLEY_TRAILER2)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER3:
	    apsr_output(" (BERKLEY_TRAILER3)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER4:
	    apsr_output(" (BERKLEY_TRAILER4)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER5:
	    apsr_output(" (BERKLEY_TRAILER5)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER6:
	    apsr_output(" (BERKLEY_TRAILER6)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER7:
	    apsr_output(" (BERKLEY_TRAILER7)");
	    break;
          case APSR_ETHERTYPE_BERKLEY_TRAILER8:
	    apsr_output(" (BERKLEY_TRAILER8)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER9:
	    apsr_output(" (BERKLEY_TRAILER9)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER10:
	    apsr_output(" (BERKLEY_TRAILER10)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER11:
	    apsr_output(" (BERKLEY_TRAILER11)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER12:
	    apsr_output(" (BERKLEY_TRAILER12)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER13:
	    apsr_output(" (BERKLEY_TRAILER13)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER14:
	    apsr_output(" (BERKLEY_TRAILER14)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER15:
	    apsr_output(" (BERKLEY_TRAILER15)");
	    break;
	  case APSR_ETHERTYPE_BERKLEY_TRAILER_IP:
	    apsr_output(" (BERKLEY_TRAILER_IP)");
	    break;
	  case APSR_ETHERTYPE_VALID_SYSTEMS:
	    apsr_output(" (VALID_SYSTEMS)");
	    break;
	  case APSR_ETHERTYPE_BASIC_BLOCK_PROTO:
	    apsr_output(" (BASIC_BLOCK_PROTO)");
	    break;
	  case APSR_ETHERTYPE_BBN_SIMNET:
	    apsr_output(" (BBN_SIMNET)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED:
	    apsr_output(" (DEC_UNASSIGNED)");
	    break;
	  case APSR_ETHERTYPE_DEC_MOP_REMOTE:
	    apsr_output(" (DEC_MOP_REMOTE)");
	    break;
	  case APSR_ETHERTYPE_DECNET_IV_ROUTE:
	    apsr_output(" (DECNET_IV_ROUTE)");
	    break;
	  case APSR_ETHERTYPE_DEC_LAT:
	    apsr_output(" (DEC_LAT)");
	    break;
	  case APSR_ETHERTYPE_DEC_CUSTOMER_PROTO:
	    apsr_output(" (DEC_CUSTOMER_PROTO)");
	    break;
	  case APSR_ETHERTYPE_DEC_LAVC_SCA:
	    apsr_output(" (DEC_LAVC_SCA)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED2:
	    apsr_output(" (DEC_UNASSIGNED2)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED3:
	    apsr_output(" (DEC_UNASSIGNED3)");
	    break;
	  case APSR_ETHERTYPE_3COM1:
	    apsr_output(" (3COM1)");
	    break;
	  case APSR_ETHERTYPE_3COM2:
	    apsr_output(" (3COM2)");
	    break;
	  case APSR_ETHERTYPE_3COM3:
	    apsr_output(" (3COM3)");
	    break;
	  case APSR_ETHERTYPE_3COM4:
	    apsr_output(" (3COM4)");
	    break;
	  case APSR_ETHERTYPE_3COM5:
	    apsr_output(" (3COM5)");
	    break;
	  case APSR_ETHERTYPE_TRANS_ETHER:
	    apsr_output(" (TRANS_ETHER)");
	    break;
	  case APSR_ETHERTYPE_RAW_FRAME_RELAY:
	    apsr_output(" (RAW_FRAME_RELAY)");
	    break;
	  case APSR_ETHERTYPE_UNGERMANN_BASS2:
	    apsr_output(" (UNGERMANN_BASS2)");
	    break;
          case APSR_ETHERTYPE_UNGERMANN_BASS3:
	    apsr_output(" (UNGERMANN_BASS3)");
	    break;
	  case APSR_ETHERTYPE_LRT1:
	    apsr_output(" (LRT1)");
	    break;
	  case APSR_ETHERTYPE_LRT2:
	    apsr_output(" (LRT2)");
	    break;
	  case APSR_ETHERTYPE_LRT3:
	    apsr_output(" (LRT3)");
	    break;
          case APSR_ETHERTYPE_LRT4:
	    apsr_output(" (LRT4)");
	    break;
	  case APSR_ETHERTYPE_LRT5:
	    apsr_output(" (LRT5)");
	    break;
	  case APSR_ETHERTYPE_LRT6:
	    apsr_output(" (LRT6)");
	    break;
	  case APSR_ETHERTYPE_LRT7:
	    apsr_output(" (LRT7)");
	    break;
	  case APSR_ETHERTYPE_LRT8:
	    apsr_output(" (LRT8)");
	    break;
	  case APSR_ETHERTYPE_LRT9:
	    apsr_output(" (LRT9)");
	    break;
          case APSR_ETHERTYPE_LRT10:
	    apsr_output(" (LRT10)");
	    break;
	  case APSR_ETHERTYPE_PROTEON:
	    apsr_output(" (PROTEON)");
	    break;
	  case APSR_ETHERTYPE_CABLETRON:
	    apsr_output(" (CABLETRON)");
	    break;
	  case APSR_ETHERTYPE_CRONUS_VLN:
	    apsr_output(" (CRONUS_VLN)");
	    break;
	  case APSR_ETHERTYPE_CRONUS_DIRECT:
	    apsr_output(" (CRONUS_DIRECT)");
	    break;
          case APSR_ETHERTYPE_HP_PROBE:
	    apsr_output(" (HP_PROBE)");
	    break;
	  case APSR_ETHERTYPE_NESTAR:
	    apsr_output(" (NESTAR)");
	    break;
          case APSR_ETHERTYPE_ATT1:
	    apsr_output(" (ATT1)");
	    break;
	  case APSR_ETHERTYPE_EXCELAN:
	    apsr_output(" (EXCELAN)");
	    break;
	  case APSR_ETHERTYPE_SGI_DIGANOSTICS:
	    apsr_output(" (SGI_DIGANOSTICS)");
	    break;
	  case APSR_ETHERTYPE_SGI_NETWORK_GAMES:
	    apsr_output(" (SGI_NETWORK_GAMES)");
	    break;
	  case APSR_ETHERTYPE_SGI_RESERVED:
	    apsr_output(" (SGI_RESERVED)");
	    break;
	  case APSR_ETHERTYPE_SGI_BOUNCE_SERVER:
	    apsr_output(" (SGI_BOUNCE_SERVER)");
	    break;
	  case APSR_ETHERTYPE_APOLLO_DOMAIN:
	    apsr_output(" (APOLLO_DOMAIN)");
	    break;
	  case APSR_ETHERTYPE_TYMSHARE:
	    apsr_output(" (TYMSHARE)");
	    break;
	  case APSR_ETHERTYPE_TIGAN:
	    apsr_output(" (TIGAN)");
	    break;
	  case APSR_ETHERTYPE_AEONIC_SYSTEMS:
	    apsr_output(" (AEONIC_SYSTEMS)");
	    break;
	  case APSR_ETHERTYPE_DEC_LANBRIDGE:
	    apsr_output(" (DEC_LANBRIDGE)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED4:
	    apsr_output(" (DEC_UNASSIGNED4)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED5:
	    apsr_output(" (DEC_UNASSIGNED5)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED6:
	    apsr_output(" (DEC_UNASSIGNED6)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED7:
	    apsr_output(" (DEC_UNASSIGNED7)");
	    break;
	  case APSR_ETHERTYPE_DEC_ETHERNET_ENC:
	    apsr_output(" (DEC_ETHERNET_ENC)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED8:
	    apsr_output(" (DEC_UNASSIGNED8)");
	    break;
	  case APSR_ETHERTYPE_DC_LAN_TRAFFIC:
	    apsr_output(" (DC_LAN_TRAFFIC)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED9:
	    apsr_output(" (DEC_UNASSIGNED9)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED10:
	    apsr_output(" (DEC_UNASSIGNED10)");
	    break;
	  case APSR_ETHERTYPE_DEC_UNASSIGNED11:
	    apsr_output(" (DEC_UNASSIGNED11)");
	    break;
	  case APSR_ETHERTYPE_PLANNING:
	    apsr_output(" (PLANNING)");
	    break;
	  case APSR_ETHERTYPE_ATT2:
	    apsr_output(" (ATT2)");
	    break;
	  case APSR_ETHERTYPE_ATT3:
	    apsr_output(" (ATT3)");
	    break;
	  case APSR_ETHERTYPE_EXPERDATA:
	    apsr_output(" (EXPERDATA)");
	    break;
	  case APSR_ETHERTYPE_STANFORD1:
	    apsr_output(" (STANFORD1)");
	    break;
	  case APSR_ETHERTYPE_STANFORD2:
	    apsr_output(" (STANFORD2)");
	    break;
	  case APSR_ETHERTYPE_EVANS_SUTHERLAND:
	    apsr_output(" (EVANS_SUTHERLAND)");
	    break;
	  case APSR_ETHERTYPE_LITTLE_MACHINES:
	    apsr_output(" (LITTLE_MACHINES)");
	    break;
	  case APSR_ETHERTYPE_COUNTERPOINT:
	    apsr_output(" (COUNTERPOINT)");
	    break;
	  case APSR_ETHERTYPE_UNI_MASS1:
	    apsr_output(" (UNI_MASS1)");
	    break;
	  case APSR_ETHERTYPE_UNI_MASS2:
	    apsr_output(" (UNI_MASS2)");
	    break;
          case APSR_ETHERTYPE_VEECO:
	    apsr_output(" (VEECO)");
	    break;
	  case APSR_ETHERTYPE_GENERAL_DYNAMICS:
	    apsr_output(" (GENERAL_DYNAMICS)");
	    break;
	  case APSR_ETHERTYPE_ATT4:
	    apsr_output(" (ATT4)");
	    break;
	  case APSR_ETHERTYPE_AUTOPHON:
	    apsr_output(" (AUTOPHON)");
	    break;
	  case APSR_ETHERTYPE_COMDESIGN:
	    apsr_output(" (COMDESIGN)");
	    break;
	  case APSR_ETHERTYPE_CMPUTGRAPHIC:
	    apsr_output(" (CMPUTGRAPHIC)");
	    break;
	  case APSR_ETHERTYPE_LANDMARK_GRAPHICS1:
	    apsr_output(" (LANDMARK_GRAPHICS1)");
	    break;
	  case APSR_ETHERTYPE_LANDMARK_GRAPHICS2:
	    apsr_output(" (LANDMARK_GRAPHICS2)");
	    break;
	  case APSR_ETHERTYPE_LANDMARK_GRAPHICS3:
	    apsr_output(" (LANDMARK_GRAPHICS3)");
	    break;
	  case APSR_ETHERTYPE_MATRA:
	    apsr_output(" (MATRA)");
	    break;
          case APSR_ETHERTYPE_DANSK_DATA:
	    apsr_output(" (DANSK_DATA)");
	    break;
	  case APSR_ETHERTYPE_MERIT:
	    apsr_output(" (MERIT)");
	    break;
	  case APSR_ETHERTYPE_VITALINK1:
	    apsr_output(" (VITALINK1)");
	    break;
	  case APSR_ETHERTYPE_VITALINK2:
	    apsr_output(" (VITALINK2)");
	    break;
	  case APSR_ETHERTYPE_VITALINK3:
	    apsr_output(" (VITALINK3)");
	    break;
	  case APSR_ETHERTYPE_COUNTERPOINT2:
	    apsr_output(" (COUNTERPOINT2)");
	    break;
	  case APSR_ETHERTYPE_COUNTERPOINT3:
	    apsr_output(" (COUNTERPOINT3)");
	    break;
	  case APSR_ETHERTYPE_COUNTERPOINT4:
	    apsr_output(" (COUNTERPOINT4)");
	    break;
	  case APSR_ETHERTYPE_APPLETALK:
	    apsr_output(" (APPLETALK)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY13:
	    apsr_output(" (DATABILITY13)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY14:
	    apsr_output(" (DATABILITY14)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY15:
	    apsr_output(" (DATABILITY15)");
	    break;
	  case APSR_ETHERTYPE_SPIDER_SYSTEMS:
	    apsr_output(" (SPIDER_SYSTEMS)");
	    break;
	  case APSR_ETHERTYPE_NIXDORF2:
	    apsr_output(" (NIXDORF2)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS1:
	    apsr_output(" (SIEMENS_GAMMASONICS1)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS2:
	    apsr_output(" (SIEMENS_GAMMASONICS2)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS3:
	    apsr_output(" (SIEMENS_GAMMASONICS3)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS4:
	    apsr_output(" (SIEMENS_GAMMASONICS4)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS5:
	    apsr_output(" (SIEMENS_GAMMASONICS5)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS6:
	    apsr_output(" (SIEMENS_GAMMASONICS6)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS7:
	    apsr_output(" (SIEMENS_GAMMASONICS7)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS8:
	    apsr_output(" (SIEMENS_GAMMASONICS8)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS9:
	    apsr_output(" (SIEMENS_GAMMASONICS9)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS10:
	    apsr_output(" (SIEMENS_GAMMASONICS10)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS11:
	    apsr_output(" (SIEMENS_GAMMASONICS11)");
	    break;
          case APSR_ETHERTYPE_SIEMENS_GAMMASONICS12:
	    apsr_output(" (SIEMENS_GAMMASONICS12)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS13:
	    apsr_output(" (SIEMENS_GAMMASONICS13)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS14:
	    apsr_output(" (SIEMENS_GAMMASONICS14)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS15:
	    apsr_output(" (SIEMENS_GAMMASONICS15)");
	    break;
	  case APSR_ETHERTYPE_SIEMENS_GAMMASONICS16:
	    apsr_output(" (SIEMENS_GAMMASONICS16)");
	    break;
	  case APSR_ETHERTYPE_DCA_CLUSTER1:
	    apsr_output(" (DCA_CLUSTER1)");
	    break;
	  case APSR_ETHERTYPE_DCA_CLUSTER2:
	    apsr_output(" (DCA_CLUSTER2)");
	    break;
	  case APSR_ETHERTYPE_DCA_CLUSTER3:
	    apsr_output(" (DCA_CLUSTER3)");
	    break;
	  case APSR_ETHERTYPE_DCA_CLUSTER4:
	    apsr_output(" (DCA_CLUSTER4)");
	    break;
	  case APSR_ETHERTYPE_BANYAN_SYSTEMS1:
	    apsr_output(" (BANYAN_SYSTEMS1)");
	    break;
	  case APSR_ETHERTYPE_BANYAN_SYSTEMS2:
	    apsr_output(" (BANYAN_SYSTEMS2)");
	    break;
	  case APSR_ETHERTYPE_PACER_SOFTWARE:
	    apsr_output(" (PACER_SOFTWARE)");
	    break;
	  case APSR_ETHERTYPE_APPLITEK:
	    apsr_output(" (APPLITEK)");
	    break;
	  case APSR_ETHERTYPE_INTERGRAPH1:
	    apsr_output(" (INTERGRAPH1)");
	    break;
	  case APSR_ETHERTYPE_INTERGRAPH2:
	    apsr_output(" (INTERGRAPH2)");
	    break;
	  case APSR_ETHERTYPE_INTERGRAPH3:
	    apsr_output(" (INTERGRAPH3)");
	    break;
	  case APSR_ETHERTYPE_INTERGRAPH4:
	    apsr_output(" (INTERGRAPH4)");
	    break;
	  case APSR_ETHERTYPE_INTERGRAPH5:
	    apsr_output(" (INTERGRAPH5)");
	    break;
	  case APSR_ETHERTYPE_HARRIS1:
	    apsr_output(" (HARRIS1)");
	    break;
	  case APSR_ETHERTYPE_HARRIS2:
	    apsr_output(" (HARRIS2)");
	    break;
	  case APSR_ETHERTYPE_TAYLOR_INSTRUMENT1:
	    apsr_output(" (TAYLOR_INSTRUMENT1)");
	    break;
          case APSR_ETHERTYPE_TAYLOR_INSTRUMENT2:
	    apsr_output(" (TAYLOR_INSTRUMENT2)");
	    break;
	  case APSR_ETHERTYPE_TAYLOR_INSTRUMENT3:
	    apsr_output(" (TAYLOR_INSTRUMENT3)");
	    break;
	  case APSR_ETHERTYPE_TAYLOR_INSTRUMENT4:
	    apsr_output(" (TAYLOR_INSTRUMENT4)");
	    break;
	  case APSR_ETHERTYPE_ROSEMOUNT1:
	    apsr_output(" (ROSEMOUNT1)");
	    break;
	  case APSR_ETHERTYPE_ROSEMOUNT2:
	    apsr_output(" (ROSEMOUNT2)");
	    break;
	  case APSR_ETHERTYPE_IBM_SNA:
	    apsr_output(" (IBM_SNA)");
	    break;
	  case APSR_ETHERTYPE_VARIAN_ASSOCIATES:
	    apsr_output(" (VARIAN_ASSOCIATES)");
	    break;
	  case APSR_ETHERTYPE_ALLEN_BRADLEY1:
	    apsr_output(" (ALLEN_BRADLEY1)");
	    break;
	  case APSR_ETHERTYPE_ALLEN_BRADLEY2:
	    apsr_output(" (ALLEN_BRADLEY2)");
	    break;
	  case APSR_ETHERTYPE_ALLEN_BRADLEY3:
	    apsr_output(" (ALLEN_BRADLEY3)");
	    break;
	  case APSR_ETHERTYPE_ALLEN_BRADLEY4:
	    apsr_output(" (ALLEN_BRADLEY4)");
	    break;
	  case APSR_ETHERTYPE_ALLEN_BRADLEY5:
	    apsr_output(" (ALLEN_BRADLEY5)");
	    break;
	  case APSR_ETHERTYPE_TRFS:
	    apsr_output(" (TRFS)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY1:
	    apsr_output(" (DATABILITY1)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY2:
	    apsr_output(" (DATABILITY2)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY3:
	    apsr_output(" (DATABILITY3)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY4:
	    apsr_output(" (DATABILITY4)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY5:
	    apsr_output(" (DATABILITY5)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY6:
	    apsr_output(" (DATABILITY6)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY7:
	    apsr_output(" (DATABILITY7)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY8:
	    apsr_output(" (DATABILITY8)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY9:
	    apsr_output(" (DATABILITY9)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY10:
	    apsr_output(" (DATABILITY10)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY11:
	    apsr_output(" (DATABILITY11)");
	    break;
	  case APSR_ETHERTYPE_DATABILITY12:
	    apsr_output(" (DATABILITY12)");
	    break;
	  case APSR_ETHERTYPE_RETIX:
	    apsr_output(" (RETIX)");
	    break;
	  case APSR_ETHERTYPE_AARP:
	    apsr_output(" (AARP)");
	    break;
	  case APSR_ETHERTYPE_KINETICS1:
	    apsr_output(" (KINETICS1)");
	    break;
	  case APSR_ETHERTYPE_KINETICS2:
	    apsr_output(" (KINETICS2)");
	    break;
	  case APSR_ETHERTYPE_APOLLO:
	    apsr_output(" (APOLLO)");
	    break;
	  case APSR_ETHERTYPE_WELLFLEET1:
	    apsr_output(" (WELLFLEET1)");
	    break;
	  /* case APSR_ETHERTYPE_WELLFLEET2:
	     apsr_output(" (WELLFLEET2)");
	     break; */
	  case APSR_ETHERTYPE_WELLFLEET3:
	    apsr_output(" (WELLFLEET3)");
	    break;
	  case APSR_ETHERTYPE_WELLFLEET_BOFL:
	    apsr_output(" (WELLFLEET BOFL)");
	    break;
	  case APSR_ETHERTYPE_WELLFLEET5:
	    apsr_output(" (WELLFLEET5)");
	    break;
	  case APSR_ETHERTYPE_SYMBOLICS_PRIVATE2:
	    apsr_output(" (SYMBOLICS_PRIVATE2)");
	    break;
          case APSR_ETHERTYPE_SYMBOLICS_PRIVATE3:
	    apsr_output(" (SYMBOLICS_PRIVATE3)");
	    break;
	  case APSR_ETHERTYPE_SYMBOLICS_PRIVATE4:
	    apsr_output(" (SYMBOLICS_PRIVATE4)");
	    break;
	  case APSR_ETHERTYPE_HAYES:
	    apsr_output(" (HAYES)");
	    break;
	  case APSR_ETHERTYPE_VG:
	    apsr_output(" (VG)");
	    break;
	  case APSR_ETHERTYPE_BRIDGE1:
	    apsr_output(" (BRIDGE1)");
	    break;
	  case APSR_ETHERTYPE_BRIDGE2:
	    apsr_output(" (BRIDGE2)");
	    break;
	  case APSR_ETHERTYPE_BRIDGE3:
	    apsr_output(" (BRIDGE3)");
	    break;
	  case APSR_ETHERTYPE_BRIDGE4:
	    apsr_output(" (BRIDGE4)");
	    break;
	  case APSR_ETHERTYPE_NOVELL:
	    apsr_output(" (NOVELL)");
	    break;
          case APSR_ETHERTYPE_KTI1:
	    apsr_output(" (KTI1)");
	    break;
	  case APSR_ETHERTYPE_KTI2:
	    apsr_output(" (KTI2)");
	    break;
	  case APSR_ETHERTYPE_KTI3:
	    apsr_output(" (KTI3)");
	    break;
	  case APSR_ETHERTYPE_KTI4:
	    apsr_output(" (KTI4)");
	    break;
	  case APSR_ETHERTYPE_KTI5:
	    apsr_output(" (KTI5)");
	    break;
	  case APSR_ETHERTYPE_LOGICRAFT:
	    apsr_output(" (LOGICRAFT)");
	    break;
	  case APSR_ETHERTYPE_NETWORK_COMPUTING:
	    apsr_output(" (NETWORK_COMPUTING)");
	    break;
	  case APSR_ETHERTYPE_ALPHA_MICRO:
	    apsr_output(" (ALPHA_MICRO)");
	    break;
	  case APSR_ETHERTYPE_SNMP:
	    apsr_output(" (SNMP)");
	    break;
	  case APSR_ETHERTYPE_BIIN1:
	    apsr_output(" (BIIN1)");
	    break;
	  case APSR_ETHERTYPE_BIIN2:
	    apsr_output(" (BIIN2)");
	    break;
	  case APSR_ETHERTYPE_ELITE_CONCEPT:
	    apsr_output(" (ELITE_CONCEPT)");
	    break;
	  case APSR_ETHERTYPE_RATIONAL_CORP:
	    apsr_output(" (RATIONAL_CORP)");
	    break;
	  case APSR_ETHERTYPE_QUALCOMM1:
	    apsr_output(" (QUALCOMM1)");
	    break;
	  case APSR_ETHERTYPE_QUALCOMM2:
	    apsr_output(" (QUALCOMM2)");
	    break;
	  case APSR_ETHERTYPE_COMPUTER_PROTOCOL1:
	    apsr_output(" (COMPUTER_PROTOCOL1)");
	    break;
	  case APSR_ETHERTYPE_COMPUTER_PROTOCOL2:
	    apsr_output(" (COMPUTER_PROTOCOL2)");
	    break;
	  case APSR_ETHERTYPE_CHARLES_RIVER1:
	    apsr_output(" (CHARLES_RIVER1)");
	    break;
	  case APSR_ETHERTYPE_CHARLES_RIVER2:
	    apsr_output(" (CHARLES_RIVER2)");
	    break;
	  case APSR_ETHERTYPE_XTP:
	    apsr_output(" (XTP)");
	    break;
          case APSR_ETHERTYPE_SGI_TIME_WARNER:
	    apsr_output(" (SGI_TIME_WARNER)");
	    break;
	  case APSR_ETHERTYPE_HIPPI_FP:
	    apsr_output(" (HIPPI_FP)");
	    break;
	  case APSR_ETHERTYPE_HIPPI_STP:
	    apsr_output(" (HIPPI_STP)");
	    break;
          case APSR_ETHERTYPE_HIPPI_64001:
	    apsr_output(" (HIPPI_64001)");
	    break;
	  case APSR_ETHERTYPE_HIPPI_64002:
	    apsr_output(" (HIPPI_64002)");
	    break;
	  case APSR_ETHERTYPE_SILLICON_GRAPHICS1:
	    apsr_output(" (SILLICON_GRAPHICS1)");
	    break;
	  case APSR_ETHERTYPE_SILLICON_GRAPHICS2:
	    apsr_output(" (SILLICON_GRAPHICS2)");
	    break;
	  case APSR_ETHERTYPE_MOTOROLA3:
	    apsr_output(" (MOTOROLA3)");
	    break;
	  case APSR_ETHERTYPE_QUALCOMM3:
	    apsr_output(" (QUALCOMM3)");
	    break;
	  case APSR_ETHERTYPE_QUALCOMM4:
	    apsr_output(" (QUALCOMM4)");
	    break;
	  case APSR_ETHERTYPE_ARAI:
	    apsr_output(" (ARAI)");
	    break;
	  case APSR_ETHERTYPE_RAD1:
	    apsr_output(" (RAD1)");
	    break;
	  case APSR_ETHERTYPE_RAD2:
	    apsr_output(" (RAD2)");
	    break;
          case APSR_ETHERTYPE_XYPLEX4:
	    apsr_output(" (XYPLEX4)");
	    break;
	  case APSR_ETHERTYPE_XYPLEX5:
	    apsr_output(" (XYPLEX5)");
	    break;
	  case APSR_ETHERTYPE_APRICOT1:
	    apsr_output(" (APRICOT1)");
	    break;
	  case APSR_ETHERTYPE_APRISOFT2:
	    apsr_output(" (APRISOFT2)");
	    break;
	  case APSR_ETHERTYPE_ARTISOFT3:
	    apsr_output(" (ARTISOFT3)");
	    break;
	  case APSR_ETHERTYPE_ARTISOFT4:
	    apsr_output(" (ARTISOFT4)");
	    break;
	  case APSR_ETHERTYPE_POLYGON1:
	    apsr_output(" (POLYGON1)");
	    break;
	  case APSR_ETHERTYPE_POLYGON2:
	    apsr_output(" (POLYGON2)");
	    break;
	  case APSR_ETHERTYPE_COMSAT1:
	    apsr_output(" (COMSAT1)");
	    break;
	  case APSR_ETHERTYPE_COMSAT2:
	    apsr_output(" (COMSAT2)");
	    break;
	  case APSR_ETHERTYPE_SAIC1:
	    apsr_output(" (SAIC1)");
	    break;
	  case APSR_ETHERTYPE_SAIC2:
	    apsr_output(" (SAIC2)");
	    break;
	  case APSR_ETHERTYPE_VG_ANALYTICAL1:
	    apsr_output(" (VG_ANALYTICAL1)");
	    break;
	  case APSR_ETHERTYPE_VG_ANALYTICAL2:
	    apsr_output(" (VG_ANALYTICAL2)");
	    break;
	  case APSR_ETHERTYPE_QUANTUM1:
	    apsr_output(" (QUANTUM1)");
	    break;
	  case APSR_ETHERTYPE_QUNATUM2:
	    apsr_output(" (QUNATUM2)");
	    break;
	  case APSR_ETHERTYPE_ASCOM1:
	    apsr_output(" (ASCOM1)");
	    break;
	  case APSR_ETHERTYPE_ASCOM2:
	    apsr_output(" (ASCOM2)");
	    break;
	  case APSR_ETHERTYPE_ENCRYPTION1:
	    apsr_output(" (ENCRYPTION1)");
	    break;
	  case APSR_ETHERTYPE_ENCRYPTION2:
	    apsr_output(" (ENCRYPTION2)");
	    break;
	  case APSR_ETHERTYPE_ATHENA1:
	    apsr_output(" (ATHENA1)");
	    break;
	  case APSR_ETHERTYPE_ATHENA2:
	    apsr_output(" (ATHENA2)");
	    break;
	  case APSR_ETHERTYPE_CHARLES_RIVER3:
	    apsr_output(" (CHARLES_RIVER3)");
	    break;
	  case APSR_ETHERTYPE_CHARLES_RIVER4:
	    apsr_output(" (CHARLES_RIVER4)");
	    break;
	  case APSR_ETHERTYPE_INST_IND1:
	    apsr_output(" (INST_IND1)");
	    break;
	  case APSR_ETHERTYPE_INST_IND2:
	    apsr_output(" (INST_IND2)");
	    break;
	  case APSR_ETHERTYPE_TAURUS1:
	    apsr_output(" (TAURUS1)");
	    break;
	  case APSR_ETHERTYPE_TAURUS2:
	    apsr_output(" (TAURUS2)");
	    break;
	  case APSR_ETHERTYPE_WALKER1:
	    apsr_output(" (WALKER1)");
	    break;
	  case APSR_ETHERTYPE_WALKER2:
	    apsr_output(" (WALKER2)");
	    break;
	  case APSR_ETHERTYPE_IDEA1:
	    apsr_output(" (IDEA1)");
	    break;
	  case APSR_ETHERTYPE_IDEA2:
	    apsr_output(" (IDEA2)");
	    break;
	  case APSR_ETHERTYPE_COMPUTER_NETWORK1:
	    apsr_output(" (COMPUTER_NETWORK1)");
	    break;
	  case APSR_ETHERTYPE_COMPUTER_NETWORK2:
	    apsr_output(" (COMPUTER_NETWORK2)");
	    break;
	  case APSR_ETHERTYPE_GATEWAY1:
	    apsr_output(" (GATEWAY1)");
	    break;
	  case APSR_ETHERTYPE_GATEWAY2:
	    apsr_output(" (GATEWAY2)");
	    break;
	  case APSR_ETHERTYPE_SECTRA:
	    apsr_output(" (SECTRA)");
	    break;
	  case APSR_ETHERTYPE_DELTA:
	    apsr_output(" (DELTA)");
	    break;
	  case APSR_ETHERTYPE_ATOMIC:
	    apsr_output(" (ATOMIC)");
	    break;
	  case APSR_ETHERTYPE_LANDIS:
	    apsr_output(" (LANDIS)");
	    break;
	  case APSR_ETHERTYPE_LANDIS2:
	    apsr_output(" (LANDIS2)");
	    break;
	  case APSR_ETHERTYPE_MOTOROLA1:
	    apsr_output(" (MOTOROLA1)");
	    break;
	  case APSR_ETHERTYPE_MOTOROLA2:
	    apsr_output(" (MOTOROLA2)");
	    break;
	  case APSR_ETHERTYPE_TCP_IP_COMPRESSION:
	    apsr_output(" (TCP_IP_COMPRESSION)");
	    break;
	  case APSR_ETHERTYPE_IP_AUTONOMOUS:
	    apsr_output(" (IP_AUTONOMOUS)");
	    break;
	  case APSR_ETHERTYPE_SECURE:
	    apsr_output(" (SECURE)");
	    break;
	  case APSR_ETHERTYPE_MPLS_UNI:
	    apsr_output(" (MPLS_UNI)");
	    break;
	  case APSR_ETHERTYPE_MPLS_MULTI:
	    apsr_output(" (MPLS_MULTI)");
	    break;
	  case APSR_ETHERTYPE_INVISIBLE1:
	    apsr_output(" (INVISIBLE1)");
	    break;
	  case APSR_ETHERTYPE_INVISIBLE2:
	    apsr_output(" (INVISIBLE2)");
	    break;
	  case APSR_ETHERTYPE_XNS2:
	    apsr_output(" (XNS2)");
	    break;
	  case APSR_ETHERTYPE_TCP_IP_SYS:
	    apsr_output(" (TCP_IP_SYS)");
	    break;
	  case APSR_ETHERTYPE_LOOP_DETECT:
	    apsr_output(" (LOOP_DETECT)");
	    break;
	  case APSR_ETHERTYPE_F000:
	    apsr_output(" (F000)");
	    break;
	  case APSR_ETHERTYPE_FFFF:
	    apsr_output(" (FFFF)");
	    break;
        default:
	  apsr_output(" (unknown type)");
	  break;
  }
}

