#include "../include/aprecv-general.h"

int print_token_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size) {
  static struct token_pkt_header *token_header;
  static unsigned char type=0, pcf=0;

  token_header = (struct token_pkt_header *) packet;
  
  apsr_output("\n- TokenRing Header -");

  if(header->caplen < APSR_TOKEN) {
	  apsr_output("\nError: Captured data length(%d bytes) < TokenRing packet header length.\n", (int) header->caplen);
	  if(statistic)
		  stats.tr.token_ring++;
	  goto end_tokenring;
  }

  if(expand_token || expand_all) {
    apsr_output("\n  Access Control: 0x%02x", token_header->ac);
    apsr_output("\n  Priority: %u", (token_header->ac & 0xe0));
    apsr_output(", Token: ");
    if(token_header->ac & 0x10)
   	 apsr_output("True");
    else 
    	 apsr_output("False");
    apsr_output("\n  Monitor: ");
    if(token_header->ac & 0x08)
         apsr_output("True");
    else
         apsr_output("False");
    apsr_output(", Reservation: %u", token_header->ac & 0x07);	    
    
    apsr_output("\n  Frame Control: 0x%02x", token_header->fc);
  }
  type = (token_header->fc & 0xe0) >> 6;
  pcf  = token_header->fc & 0x0f;
  if(expand_token || expand_all) {
    apsr_output("\n  Type: %u", type);
    if(verbose) {
	    switch(type) {
		    case APSR_TOKEN_FCTYPE_MAC:
		      apsr_output(" (MAC)");
		      break;
		    case APSR_TOKEN_FCTYPE_LLC:
		      apsr_output(" (LLC)");
		      break;
		    case APSR_TOKEN_FCTYPE_RESER:
		      apsr_output(" (Reserved)");
		      break;
		    default:
		      apsr_output(" (unknown)");
		      break;
	    }
    }
    apsr_output(", PCF: %u", pcf);
    if(verbose) {
            switch(pcf) {
		    case APSR_TOKEN_FCPCF_NORMAL:
		      apsr_output(" (Normal buffer)");
		      break;
		    case APSR_TOKEN_FCPCF_EXPRES:
		      apsr_output(" (Express buffer)");
		      break;
		    case APSR_TOKEN_FCPCF_PURGE:
		      apsr_output(" (Purge)");
		      break;
		    case APSR_TOKEN_FCPCF_CLAIM:
		      apsr_output(" (Claim token)");
		      break;
		    case APSR_TOKEN_FCPCF_BEACON:
		      apsr_output(" (Beacon)");
		      break;
		    case APSR_TOKEN_FCPCF_ACTIVM:
		      apsr_output(" (Active monitor present)");
		      break;
		    case APSR_TOKEN_FCPCF_STANDM:
		      apsr_output(" (Standby monitor present)");
		      break;
		    default:
		      apsr_output(" (unknown)");
		      break;
	    }
    }
				
    apsr_output("\n  MAC Dst: %s", ether2str(token_header->destination));
    if(verbose)
         resolve_mac(token_header->destination);
	 
    apsr_output("\n  MAC Src: %s", ether2str(token_header->source));
    if(verbose)
         resolve_mac(token_header->source);
		 
    apsr_output("\n  Src Routed: ");
    if(token_header->source[0] & 128) 
 	 apsr_output("True");
    else 
         apsr_output("False");
  }

  if(statistic) 
  	  stats.c.token_ring++;

  if(type == APSR_TOKEN_FCTYPE_LLC && !(token_header->source[0] & 128)) 
	  print_llc_packet(header, packet, opt_size, header->caplen);

end_tokenring:
  apsr_output("\n");

  return(RET_SUCCESS);
}
