#include "../include/aprecv-general.h"

int print_udp_packet (struct aprecv_frame_header *header, unsigned char *packet, int opt_size, int ip_version) {
  static struct udp_pkt_header *udp_header;
  static unsigned short src_port, dst_port, calulated_checksum;

  udp_header = (struct udp_pkt_header *)(packet+data_link_size+opt_size);
  src_port = apsr_ntohs(udp_header->src_port);
  dst_port = apsr_ntohs(udp_header->dst_port);
  #ifdef HAVE_APSRLIB_MODULE_SUPPORT
  module_length = apsr_ntohs(udp_header->length);
  #endif
  
  apsr_output("- UDP Header -\n  Src Port: %d",(int) src_port);
  if(verbose) 
	  apsr_output(" (%s)", apsr_resolve_port(src_port, "udp"));

  apsr_output(", Dst Port: %d",(int) dst_port);
  if(verbose)
          apsr_output(" (%s)", apsr_resolve_port(dst_port, "udp"));
  
  #ifdef HAVE_APSRLIB_MODULE_SUPPORT
  apsr_output("\n  Length: %d, Checksum: 0x%04x", 
		  module_length, apsr_ntohs(udp_header->sum));
  #else 
  apsr_output("\n  Length: %d, Checksum: 0x%04x", 
		  apsr_ntohs(udp_header->length), apsr_ntohs(udp_header->sum));
  #endif
 
 
  if(calc_checksums && ip_version == 4) {
	  calulated_checksum = apsr_udpv4_checksum(udp_header);
	  if(calulated_checksum == 0) 
		  apsr_output(" (correct checksum)\n");
	  else {
		  apsr_output(" (incorrect checksum - should be 0x%04x)\n",
				  calulated_checksum);
	          if(statistic)
			  stats.cr.udp++;
	  }
  }
  else 
	  apsr_output("\n");
  
  if(src_port == APSR_UDP_PORT_RIP || dst_port == APSR_UDP_PORT_RIP) {
      print_rip_packet(packet, opt_size+sizeof(struct udp_pkt_header));
  }
  else if(src_port == APSR_UDP_PORT_RIPNG || dst_port == APSR_UDP_PORT_RIPNG) {
      print_ripng_packet(packet, opt_size+sizeof(struct udp_pkt_header));
  }	  
  #ifdef HAVE_APSRLIB_MODULE_SUPPORT
  else if(module_name || modules_in_path)
      find_module(packet+data_link_size+APSR_UDP_H+opt_size, AF_INET, APSR_PROTO_UDP, APSR_PROTO_UDP, src_port, dst_port);	  
  #endif

  else if(print_payload_udp_hex && !print_payload_udp_text) {
	  print_raw_in_hex(packet+data_link_size+sizeof(struct udp_pkt_header)+opt_size, header->len-sizeof(struct udp_pkt_header)-data_link_size-opt_size);
  }
  else if(!print_payload_udp_hex && print_payload_udp_text) {
	  print_raw_in_text(packet+data_link_size+sizeof(struct udp_pkt_header)+opt_size, header->len-sizeof(struct udp_pkt_header)-data_link_size-opt_size);
  }

  if(statistic) 
        stats.c.udp++;
  
  return(RET_SUCCESS);
}
