#include "../include/aprecv-general.h"
#include "../include/aprecv-apsr.h"

int start_sniffing_apsr(char *arg, int promisc, int snaplen, int max_packets) {
  struct apsr_link_layer *apsr_ll;	
  int fd;

  /* Check user ID */
  apsr_check_uid();

  if(promisc) {
          /* Open socket */
          fd=apsr_open_io_socket();
		  
	  apsr_toggle_promisc(device, fd);

  	  /* Close socket */
          apsr_close_io_socket(fd);
  }
	  
  /* Open a link layer interface. */
  apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);
  link_layer = apsr_ll->type;

  apsr_output("Receive packet(s) on device: %s\n", device);
  
  switch (link_layer) {
	  case APSR_DLT_NULL:
	  #ifdef __OSTYPE_OPENBSD
	  #ifdef APSR_DLT_LOOP
	  case APSR_DLT_LOOP:			/* OpenBSD only */
	  #endif
	  #endif
      	    data_link_size = 4;
	    index_number = 6;
            break;
          case APSR_DLT_PPP:			/* PPP */
	    data_link_size = 4;
	    index_number = 1;
	    break;
          case APSR_DLT_ATM_RFC1483:            /* ATM RFC1483 PDU (LLC) */
            data_link_size = 0;                 /* No need */
            index_number = 10;
            break;
          case APSR_DLT_ATM_CLIP:               /*Linux Classical-IP over ATM*/
            data_link_size = 0;                 /* No need */
            index_number = 11;
            break;
	  case APSR_DLT_SUNATM:			/* SunATM */
	    data_link_size = 0;
	    index_number = 13;
	    break;
          case APSR_DLT_EN10MB:			/* Ethernet II */
          case APSR_DLT_EN3MB:			/* Experimental Ethernet */
	    data_link_size = apsr_ll->offset;
	    index_number = 0;
	    break;
	  case APSR_DLT_IEEE802:		/* TokenRing(802.5) */
            data_link_size = 14;
            index_number = 8;
            break;
	  #ifdef APSR_DLT_PPP_ETHER
	  case APSR_DLT_PPP_ETHER:		/* PPPoE / UNTESTED */
	    data_link_size = 14;
	    index_number = 2;
	    break;
	  #endif
          case APSR_DLT_C_HDLC:                 /* Cisco HDLC */
            data_link_size = 4;
            index_number = 12;
            break;
          #ifdef APSR_DLT_IEEE802_11
	  case APSR_DLT_IEEE802_11:			
	    data_link_size = 0;			/* No need */
	    index_number = 4;
	    break; 
          #endif
	  #ifdef APSR_DLT_PRISM_HEADER		/* Since Libpcap 0.7.1 */
	  case APSR_DLT_PRISM_HEADER:	
	    data_link_size = 144;		
	    index_number = 5;
	    break;
          #endif
          case APSR_DLT_PPP_BSDOS:			
	    data_link_size = 24;
 	    break;
          case APSR_DLT_SLIP:
 	    data_link_size = 16;		/* Slip */
	    index_number = 7;
	    break;
	  case APSR_DLT_SLIP_BSDOS:
	    #if __OSTYPE_OPENBSD
            data_link_size = 16;		/* Free/Open/Net/BSD/+i only */
	    #else
	    data_link_size = 24;
	    #endif
	    break;
          case APSR_DLT_FDDI:
	    data_link_size = 21;		/* FDDI */
	    index_number = 9;
	    break;
          #ifdef APSR_DLT_RAW			/* Not supported in older 
	 					  libpcap versions and archs */
	  case APSR_DLT_RAW:
	    data_link_size = 0;
	    index_number = 6;
	    break;
	  #endif
	  #ifdef APSR_DLT_LINUX_SLL			/* Not suppport in older
						  libpcap versions */
	  case APSR_DLT_LINUX_SLL:
	    data_link_size = 0;		
	    index_number = 3;
	    break;
	  #endif 
	  default:
	    apsr_stream_output((FILE *) stderr, "Error: Unknown device/datalink type");
	    #ifdef APRECV_DEBUG
	    apsr_output("(%d)", link_layer);
 	    #endif
	    apsr_output(".\n\n");
	    exit(EXIT_FAILURE);
  }
  #ifdef APRECV_DEBUG
  apsr_output("Use datalink type %d.\n", link_layer);
  #endif
     
  apsr_read_link_layer(apsr_ll, snaplen, max_packets, (apsr_sniff_callback)recv_packet_apsr);

  return(RET_SUCCESS);
}

int recv_packet_apsr(struct sockaddr *sa, unsigned char *packet, unsigned int len) {
  struct timeval ts;
  static unsigned int sec;
  static unsigned long diff_receive_time;
  static float utilization;
  time_t now_receive_time;
  struct aprecv_frame_header aprecv_header;
 
  stats.c.all++;
  
  if(beep_on)
	  apsr_output("\a");

  all_counted_bytes += len;

  /* print out the time when the packet comes in & utilization */
  /* FIXME: captured length/len */
  if(gettimeofday(&(ts), NULL) < 0) {
	  apsr_stream_output((FILE *) stderr, "Error: Can't get time.\n");  
	  #ifdef APRECV_DEBUG
	  apsr_stream_output((FILE *) stderr, "Function gettimeofday failed while receive packet, error code: %s.\n", strerror(errno));
	  #endif
  }  
  sec = ts.tv_sec % 86400;
  apsr_output("\n%02d:%02d:%02d.%06u - Received frame %lu(%u bytes length)...", sec / 3600, (sec % 3600) / 60, sec % 60, ts.tv_usec, stats.c.all, len);

  if(statistic) {
	  now_receive_time = time(NULL);
	  diff_receive_time = difftime(now_receive_time, start_receive_time);
	  if(diff_receive_time > 0) {	/* Needed, overwhise core dump */
		  utilization = all_counted_bytes / diff_receive_time;
                  if(utilization > 1024000) { /* MB */
			  utilization = utilization / 1024000;
		          apsr_output(" Utilization: %.0f MB/s", utilization);
                  }
                  if(utilization > 1024) { /* KB */
                          utilization = utilization / 1024;
                          apsr_output(" Utilization: %.0f KB/s", utilization);
                  }
                  else
                          apsr_output(" Utilization: %.0f B/s", utilization);
	  }
	  else {
		  apsr_output(" Utilization: n.a.");
          }
  }
  apsr_output("\n");

  /* Map to aprecv_frame_header */
  aprecv_header.caplen = len; /* FIXME see above */
  aprecv_header.len    = len;
  
  if(!no_dissection)
	  (*aprecv_print_index_apsr[index_number])(&aprecv_header, packet, 0);
  
  if(print_raw_hex && !print_payload_ip_hex && !print_payload_pppoe_hex && 
     !print_payload_tcp_text && !print_payload_udp_text && 
     !print_payload_tcp_hex && !print_payload_udp_hex) {
	 print_raw_in_hex(packet, len-1);
  }

  if(server_addr)
	send_server_got_packet(packet, len);
	 
  if(fflush(stdout) == EOF) {
 	 #ifdef APRECV_DEBUG
	 apsr_stream_output((FILE *) stderr, "Error: fflush(stdout) %s.\n", strerror(errno));
	 #endif
  }
 
  return(RET_SUCCESS);  
}
