#include "../include/apsrstack-general.h"

void process_ethernet_packet (struct apsrstack_frame_header *old_header, void *packet, int opt_size) {
  struct ethernet_pkt_header *ethernet_header;
  struct apsrstack_frame_header new_header;
  unsigned short ethernet_type=0; 

  ethernet_header = (struct ethernet_pkt_header *)(packet+opt_size);
  ethernet_type = apsr_ntohs(ethernet_header->type);

  new_header.caplen     = old_header->len;
  new_header.len        = old_header->len;
  memcpy(new_header.src_mac, ethernet_header->destination, APSR_ETHER_ADDR_LEN);
  memcpy(new_header.dst_mac, ethernet_header->source, APSR_ETHER_ADDR_LEN);
  new_header.type       = ethernet_type;
  
  if(new_header.caplen < APSR_ETH) {
	  /* Captured data length < Ethernet packet header length */
	  stats.ethernet.drops++; /* Drops++ */
	  return;
  }
  
  if(ethernet_type < 1500) {
	  /* Not Ethernet II or 802.x/SNAP frames */
	  stats.ethernet.drops++; /* Drops++ */
	  return;
  }	  
 
  switchto_ethernet_proto(&new_header, packet, opt_size, ethernet_type);
}

void switchto_ethernet_proto(struct apsrstack_frame_header *header, void *packet, int opt_size, unsigned short protocol) {
  switch(protocol) {
	  case APSR_ETHERTYPE_ARP:
	    process_arp_packet(packet, opt_size);
	    stats.ethernet.processed++;
	    break;
	  case APSR_ETHERTYPE_IP:
	    process_ipv4_packet(header, packet, opt_size);
	    stats.ethernet.processed++;
	    break;
          default:
	    #ifdef APSRSTACK_DEBUG
//	    apsr_output("DEBUG: Unknown Ethernet type: 0x%04x\n", protocol);
	    #endif
	    stats.ethernet.unknown_type++; /* Unknown Ethernet type++ */
            break;
  }

  return;
}

