#include "../include/apsrstack-general.h"
#include "../include/apsrstack-apsr.h"

int start_sniffing_apsr(int promisc, int snaplen, int max_packets) {
  struct apsr_link_layer *apsr_ll;	
  int fd;

  /* Check user ID */
  apsr_check_uid();

  if(promisc) {
          /* Open socket */
          fd=apsr_open_io_socket();
		  
	  apsr_toggle_promisc(device, fd);

  	  /* Close socket */
          apsr_close_io_socket(fd);
  }
	  
  /* Open a link layer interface. */
  apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

  apsr_output("Receive packet(s) on device: %s\n", device);
  
  switch (apsr_ll->type) {
          case APSR_DLT_EN10MB:			/* Ethernet II */
          case APSR_DLT_EN3MB:			/* Experimental Ethernet */
	    data_link_size = apsr_ll->offset;
	    index_number = 0;
	    break;
	  default:
	    apsr_stream_output((FILE *) stderr, "Error: Unknown device/datalink type");
	    #ifdef APSRSTACK_DEBUG
	    apsr_output("(%d)", apsr_ll->type);
 	    #endif
	    apsr_output(".\n\n");
	    exit(EXIT_FAILURE);
  }
  #ifdef APSRSTACK_DEBUG
  apsr_output("Use datalink type %d.\n", apsr_ll->type);
  #endif
     
  apsr_read_link_layer(apsr_ll, snaplen, max_packets, (apsr_sniff_callback)recv_packet_apsr);

  return(RET_SUCCESS);
}

int recv_packet_apsr(struct sockaddr *sa, unsigned char *packet, unsigned int len) {
  struct apsrstack_frame_header apsrstack_header;
  
  /* Map to apsrstack_frame_header */
  apsrstack_header.caplen = len; /* FIXME see above */
  apsrstack_header.len    = len;
  
  (*apsrstack_process_index_apsr[index_number])(&apsrstack_header, packet, 0);
  
  return(RET_SUCCESS);  
}
