/* Global buffer. */
unsigned char **buf;


/* Checksum routines */
#define APSR_CKSUM(x) (x = (x >> 16)+(x & 0xffff), (~(x+(x >> 16)) & 0xffff))
	
/* extract 24 bits from a char into an integer */
#define APSR_24TO32(x)((unsigned int)*((const unsigned char *)(x) + 0) << 16|\
 	              (unsigned int)*((const unsigned char *)(x) + 1) << 8 | \
		      (unsigned int)*((const unsigned char *)(x) + 2))

/* Header sizes */
#define APSR_MAX_PACKET_SIZE	0xFFFF		/* Maximum packet size. */
#define APSR_TOKEN		0xE		/* 14 Bytes TokenRing header */
#define APSR_TOKEN_H		APSR_TOKEN
#define APSR_ETH		0xE		/* 14 Bytes EthernetII header */
#define APSR_ETH_H		APSR_ETH
#define APSR_ETH_SNAP		0x10		/* 10 Bytes LLC/SNAP header */
#define APSR_ETH_SNAP_H		APSR_ETH_SNAP
#define APSR_FDDI		0xD		/* 13 Bytes FDDI header */
#define APSR_FDDI_H		APSR_FDDI	
#define APSR_ARP		0x1C		/* 28 Bytes ARP header */
#define APSR_ARP_H		APSR_ARP
#define APSR_IPv4		0x14		/* 20 Bytes IPv4 header */
#define APSR_IPv4_H		APSR_IPv4
#define APSR_IPv6		0x28		/* 40 Bytes IPv6 header */
#define APSR_IPv6_H		APSR_IPv6
#define APSR_TCP        	0x14    	/* 20 Bytes TCP header */
#define APSR_TCP_H      	APSR_TCP
#define APSR_UDP		0x8		/*  8 Bytes UDP header */
#define APSR_UDP_H		APSR_UDP
#define APSR_ICMPv4		0x4             /*  4 Bytes ICMPv4 header */
#define APSR_ICMPv4_H 		APSR_ICMPv4	
#define APSR_PPPoE		0x6		/*  6 Bytes PPPoE header */
#define APSR_PPPoE_H		APSR_PPPoE
#define APSR_PPP		0x2		/*  2 Bytes PPP header */
#define APSR_PPP_H		APSR_PPP
#define APSR_CBT		0x4		/*  4 Bytes CBT header */
#define APSR_CBT_H		APSR_CBT
#define APSR_GGP		0x4		/*  4 Bytes GGP header */
#define APSR_GGP_H		APSR_GGP
#define APSR_IPAH		0xC		/* 12 Bytes IPAH header (??) */
#define APSR_IPAH_H		APSR_IPAH
#define APSR_IPESP              0x8             /*  8 Bytes IPESP header */
#define APSR_IPESP_H            APSR_IPESP	
#define APSR_EGP		0xA		/* 10 Bytes EGP header (??) */
#define APSR_EGP_H		APSR_EGP
#define APSR_IGRP		0xC		/* 12 Bytes IGRP header (??) */
#define APSR_IGRP_H		APSR_IGRP
#define APSR_EIGRP              0x14            /* 20 Bytes EIGRP header */
#define APSR_EIGRP_H            APSR_EIGRP
#define APSR_IGMPv2		0x8		/*  8 Bytes IGMPv2 header */
#define APSR_IGMPv2_H		APSR_IGMPv2
#define APSR_RIPng		0x4		/*  4 Bytes RIPng header */
#define APSR_RIPng_H		APSR_RIPng
#define APSR_RIP		0x16		/* 22 Bytes RIPv1/v2 header */
#define APSR_RIP_H		APSR_RIP
#define APSR_NARP		0x12		/* 18 Bytes NARP header (??) */
#define APSR_NARP_H		APSR_NARP
#define APSR_GRE		0x4		/*  4 Bytes GRE header */
#define APSR_GRE_H		APSR_GRE
#define APSR_IFMP		0x4		/*  4 Bytes IFMP header (??) */
#define APSR_IFMP_H		APSR_IFMP
#define APSR_PIM		0x4		/*  4 Bytes PIM header */
#define APSR_PIM_H		APSR_PIM
#define APSR_IRTP               0x10            /* 10 (??) Bytes IRTP header */
#define APSR_IRTP_H             APSR_IRTP
#define APSR_CDP                0x4             /*  4 Bytes CDP header */
#define APSR_CDP_H              APSR_CDP
#define APSR_SCTP               0x6             /*  6 Bytes SCTP header */
#define APSR_SCTP_H             APSR_SCTP
#define APSR_CIP		0x12		/* 18 Bytes CTP header */
#define APSR_CIP_H		APSR_CIP

/* Protocol numbers and IPv6 extension headers. */
#define APSR_PROTO_IP		0	/* Internet protocol version 4 (IPv4) */
#define APSR_PROTO_IPV4		0	/* Internet protocol version 4 (IPv4) */
#define APSR_PROTO_ICMP		1	/* Internet control message protocol 4*/
#define APSR_PROTO_ICMPV4	1	/* Internet control message protocol 4*/
#define APSR_PROTO_IGMP		2	/* Internet group multicast protocol  */
#define APSR_PROTO_GGP		3	/* Gateway-gateway protocol           */
#define APSR_PROTO_IP_IN_IP	4	/* IP in IP Encapsulation 	      */
#define APSR_PROTO_TCP		6	/* Transmission control protocol      */
#define APSR_PROTO_CBT		7	/* Core Based Trees 		      */
#define APSR_PROTO_EGP		8	/* Exterior gateway protocol          */
#define APSR_PROTO_IGRP		9	/* IGRP		                      */
#define APSR_PROTO_PUP		12	/* PARC universal packet protocol     */
#define APSR_PROTO_UDP		17	/* User datagram protocol             */
#define APSR_PROTO_TMUX		18	/* TMux (rfc1692)		      */
#define APSR_PROTO_HMP		20	/* Host monitoring protocol           */
#define APSR_PROTO_IDP		22	/* ???                                */
#define APSR_PROTO_RDP		27	/* "reliable datagram" protocol       */
#define APSR_PROTO_IRTP         28      /* Internet Reliable Trans. Protocol  */
#define APSR_PROTO_ISO_TP4	29	/* ISO transport protocol class 4     */
#define APSR_PROTO_NETBLK	30	/* NETBLK 			      */

#define APSR_PROTO_HOPOPT	0	/* Hop-by-hop options for IPv6        */
#define APSR_PROTO_IPV6		41	/* Internet protocol version 6 (IPv6) */
#define APSR_PROTO_SDRP		42	/* Source Demand Routing Protocol     */
#define APSR_PROTO_IPV6_ROUTE	43	/* IPv6 routing header                */
#define APSR_PROTO_IPV6_FRAG	44	/* IPv6 fragment header               */
#define APSR_PROTO_RSVP		46	/* Resource ReSerVation Protocol      */
#define APSR_PROTO_GRE          47      /* Generic Routing Encapsulation      */
#define APSR_PROTO_IPV6_CRYPT	50	/* IPv6 encryption header  /  ESP     */
#define APSR_PROTO_IPV6_AUTH	51	/* IPv6 authentication header         */
#define APSR_PROTO_NARP		54	/* NARP 			      */
#define APSR_PROTO_ICMPV6	58	/* Internet control message protocol 6*/
#define APSR_PROTO_IPV6_NONXT	59	/* IPv6 no next header                */
#define APSR_PROTO_IPV6_OPTS	60	/* IPv6 destination options           */

#define APSR_PROTO_ISO_IP	80	/* ISO internet protocol              */
#define APSR_PROTO_EIGRP	88	/* EIGRP			      */
#define APSR_PROTO_OSPF		89	/* OSPF 			      */
#define APSR_PROTO_MTP		92	/* MTP				      */
#define APSR_PROTO_ETHERIP	97	/* EtherIP (rfc 3378)		      */
#define APSR_PROTO_ENCAP	98	/* RFC1241 encapsulation              */
#define APSR_PROTO_IFMP		101	/* Ipsilon Flow Management Protocol   */
#define APSR_PROTO_PIM		103	/* PIM 				      */
#define APSR_PROTO_IPCOMP	108	/* Compression Header Protocol	      */
#define APSR_PROTO_VRRP		112	/* Virtual Router Redundancy Protocol */
#define APSR_PROTO_PGM		113	/* Pragmatic General Multicast */
#define APSR_PROTO_SCTP		132	/* SCTP			              */
#define APSR_PROTO_RAW		255	/* RAW IP interface                   */

#ifndef ARPHRD_HDLC             /* From Linux 2.2.13 */
#define ARPHRD_HDLC             513
#endif

#ifndef ARPHRD_RAWHDLC
#define ARPHRD_RAWHDLC          518
#endif

#ifndef ARPHRD_FDDI             /* From Linux 2.2.13 */
#define ARPHRD_FDDI             774
#endif

#ifndef ARPHRD_SIT              /* From Linux 2.2.13 */
#define ARPHRD_SIT              776
#endif

#ifndef ARPHRD_IEEE802_TR       /* From Linux 2.4 */
#define ARPHRD_IEEE802_TR       800
#endif

#ifndef ARPHRD_IEEE80211        /* From Linux 2.4.6 */
#define ARPHRD_IEEE80211        801
#endif

#ifndef ARPHRD_IEEE80211_PRISM  /* From Linux 2.4.18 */
#define ARPHRD_IEEE80211_PRISM  802
#endif
