#include "../include/apsrlib-config.h"

/* Create a IPv4 packet. */
int apsr_ipv4 (unsigned char version, unsigned char header_length,
	       unsigned short len, unsigned int tos, unsigned short id,
	       unsigned short frag, unsigned char ttl, unsigned char proto,
	       unsigned long src, unsigned long dst, unsigned short sum,
	       struct ipv4_option *options, unsigned short option_size,
	       const unsigned char *payload, int payload_size,
	       unsigned char *buf)
{
	static struct ipv4_pkt_header ipv4_hdr;

	/* FIXME: QNX and AIX support for IP options! */
	#if !defined(__OSTYPE_QNX) && !defined(__OSTYPE_AIX) && !defined(__OSTYPE_FREEBSD) && !defined(__OSTYPE_NETBSD) && !defined(__OSTYPE_OPENBSD)
	static struct ipv4_pkt_header *op_ipv4_hdr;
	static unsigned short op_s, i;
	static unsigned char *pkt, counter;
	#endif

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
	  ERROR("apsr_ipv4: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	/* IP version + HL */
	ipv4_hdr.ipvhl = ((version << 4) | 0x0f) & ((header_length) | 0xf0);	        ipv4_hdr.type_service	= tos;			/* IP tos */
	ipv4_hdr.packet_len	= apsr_htons(APSR_IPv4+len);	/* IP total length */
	ipv4_hdr.ident		= apsr_htons(id);		/* IP ID */
	ipv4_hdr.fragment	= apsr_htons(frag);		/* Frag flags */
	ipv4_hdr.time_live	= ttl;			/* Time to live (TTL) */
	ipv4_hdr.protocol	= proto;		/* Transport protocol */
        ipv4_hdr.src_ip.s_addr  = src;                  /* Source IP */
        ipv4_hdr.dst_ip.s_addr  = dst;                  /* Destination IP */
	
	if(sum == 0) {	/* If sum = 0, apsrlib calculates it */
		ipv4_hdr.sum = 0x0000;
		ipv4_hdr.sum = in_cksum((unsigned short *)&ipv4_hdr, header_length*4, 0);
        }
	else {
		ipv4_hdr.sum = apsr_htons(sum);
	}

	
	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_IPv4, payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&ipv4_hdr,
	       sizeof(ipv4_hdr));


	/* FIXME: QNX and AIX support for IP options! */
	#if !defined(__OSTYPE_QNX) && !defined(__OSTYPE_AIX) && !defined(__OSTYPE_FREEBSD) && !defined(__OSTYPE_NETBSD) && !defined(__OSTYPE_OPENBSD)
			
	/* IPv4 options  */
	if(options && (option_size > 0)) {
	  op_ipv4_hdr = (struct ipv4_pkt_header *)(buf);
	  op_s = APSR_IPV4_HEADERLENGTH(op_ipv4_hdr);

	  /* Check if options exceed maximum packet size. */
	  if(op_s + option_size > APSR_MAX_PACKET_SIZE) {
	    #ifdef APSRLIB_DEBUG
	    ERROR("apsr_ipv4: option_size > APSR_MAX_PACKET_SIZE");
	    #endif
	    return(RET_FAILURE);
	  }

	  /* Copy the options list to the buffer (unchecked heap memory!) */
	  pkt = (unsigned char *) op_ipv4_hdr + APSR_IPv4;
	  memcpy(pkt, options->ipopt_list, option_size);
	  
	  /* Count the number of 32 bit words in the option list. */
	  /* Padding if neccessary.				 */
	  for(counter=0, i=0; counter < option_size; counter++)
	    (counter % 4) ? i : i++;

	  header_length+=i;
	  op_ipv4_hdr->ipvhl = ((version << 4) | 0x0f) & (header_length | 0xf0);
	  op_ipv4_hdr->packet_len = apsr_htons((i + op_s)*4);

	  if(sum == 0) {  /* If sum = 0, apsrlib calculates it */
		op_ipv4_hdr->sum = 0;
                op_ipv4_hdr->sum = in_cksum((unsigned short *)op_ipv4_hdr, header_length*4, 0);
          }
          else {
                op_ipv4_hdr->sum = sum;
          }
	}
	#endif

  return(RET_SUCCESS);
}
