#include "../include/apsrlib-config.h"

/* Create a IPv6 packet. */
int apsr_ipv6 (unsigned char version, unsigned char priority,
	       unsigned int flow_label, unsigned short plength,
	       unsigned char next_header, unsigned char hop,
	       struct in6_addr src_ip, struct in6_addr dst_ip,
	       const unsigned char *payload, int payload_size,
	       unsigned char *buf)
{
	static struct ipv6_pkt_header ipv6_hdr;

	if(!buf) {
	  #ifdef APSRLIB_DEBUG
	  ERROR("apsr_ipv6: !buf");
	  #endif
	  return(RET_FAILURE);
	}

	/* IP version + priority */
	ipv6_hdr.vpr = ((version << 4) | 0x0f) & ((priority) | 0xf0);	       
	ipv6_hdr.flow_label	= flow_label;		/* Flow label*/
	ipv6_hdr.payload_len	= apsr_htons(plength);	/* Payload length */
	ipv6_hdr.next_header	= next_header;		/* Next header */
	ipv6_hdr.hop_limit	= hop;			/* Hop limit */
	ipv6_hdr.src_ip		= src_ip;		/* Source IP */
        ipv6_hdr.dst_ip		= dst_ip;               /* Destination IP */
	
	/* Check the payload size and copy it to the buffer. */
	if(payload && payload_size) {
	  memcpy(buf + APSR_IPv6, payload, payload_size);
	}

	memcpy((unsigned char *)buf, (unsigned char *)&ipv6_hdr,
	       sizeof(ipv6_hdr));

  	return(RET_SUCCESS);
}
