#include <stdio.h>
#include <string.h>
#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="eth0";
	unsigned char src_mac[6] = {0x0d, 0x0e, 0x0a, 0x0d, 0x00, 0x00};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	unsigned char src_ip[4]  = {0x0a, 0x00, 0x00, 0x02};
	unsigned char dst_ip[4]  = {0x0a, 0x00, 0x00, 0x01};

	int my_ret, written;
	unsigned char *buf;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header and */
	/* 28 bytes for the arp header. */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_ARP_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       ETHERTYPE_ARP,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

	/* Secondly we construct the ARP header (APSR_ETHERTYPE_ARP). */
	my_ret = apsr_arp(APSR_ARPHRD_ETHER,	// Hardware address type
		 	  APSR_ETHERTYPE_IP,	// Protocol address type
		  	  6,			// Hardware address length
		 	  4,			// Protocol address length
		 	  APSR_ARPOP_REQUEST,	// Opcode
		 	  src_mac,		// Source hardware address
		 	  src_ip,		// Source protocol address
		 	  dst_mac,		// Destination hardware address
		 	  dst_ip,		// Destination protocol address
		 	  NULL,			// Payload
		 	  0,			// Payload size
		 	  buf + APSR_ETH_H);	// Buffer
	
	if(my_ret == -1) ERROR("apsr_arp failed!");

	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_ARP_H, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

