#include <stdio.h>
#include <string.h>
#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="lo";
	unsigned char src_mac[6] = {0x0d, 0x0e, 0x0a, 0x0d, 0x00, 0x00};
	unsigned char dst_mac[6] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};

	int my_ret, written;
	unsigned char *buf;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header,  */
	/* 20 bytes for the IPv4 header and 4 bytes for the PIM header. */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_IPv4_H + APSR_PIM_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       ETHERTYPE_IP,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

 	my_ret = apsr_ipv4(4,			// IP version
			   5,			// IP header length
			   0,			// IP length
			   0x1111,		// IP Type of Service (ToS)
			   45360,		// IP Identification (ID)
			   0x4000,		// IP Fragementation offset
			   64,			// IP Time to live (TTL)
			   APSR_PROTO_PIM,	// IP transport layer protocol
			   inet_addr("127.0.0.1"), // Source IP
			   inet_addr("127.0.0.2"), // Destination IP
			   0,			// IP checksum
			   NULL,		// IP options
			   0,			// IP option size
			   NULL,		// IP payload
			   0,			// IP payload length
			   buf + APSR_ETH_H);	// Packet buffer

	if(my_ret == -1) ERROR("apsr_ipv4 failed!");

	my_ret = apsr_pim(2,			// PIM version
			  APSR_PIMV2_TYPE_GRAFT_ACK,	// PIM type
			  0,			// PIM reserved flag
			  1234,			// PIM sequence number
			  NULL,			// PIM payload
			  0,			// PIM payload size
			  buf + APSR_ETH_H +
			  APSR_IPv4_H);		// Packet buffer

	/* Calculate the checksum */
	apsr_checksum(buf+APSR_ETH_H, APSR_PROTO_PIM, APSR_PIM_H);

	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_IPv4_H + APSR_PIM_H, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

