#include <stdio.h>
#include <string.h>
#include <apsrlib/apsrlib.h>

int main (void) {
	struct apsr_link_layer *apsr_ll;
	char *device="eth0";
	unsigned char dst_mac[6] = {0x00, 0x90, 0x1a, 0x40, 0x05, 0x28};
	unsigned char src_mac[6] = {0x52, 0x54, 0x05, 0xf2, 0xe0, 0xb3};

	int my_ret, written;
	unsigned char *buf;

	/* Open a link layer interface. */
	apsr_ll = (struct apsr_link_layer *) apsr_open_link_layer(device);

	/* Initialize the packet memory. */
	/* We need 14 bytes for the ethernet header and */
	/* 6 bytes for the PPPoE header. */
	my_ret = apsr_init_psize(APSR_ETH_H + APSR_PPPoE_H + APSR_PPP_H, &buf);

	if(my_ret == 0 || buf == NULL) ERROR("apsr_init_psize failed!");

	/* Build test packet */
	/* Firstly we construct the ethernet header. */
	my_ret = apsr_ethernet(dst_mac,		// Destination address
		     	       src_mac,		// Source address
			       APSR_ETHERTYPE_PPPOE_S,	// Ethernet type
		   	       NULL,		// Payload
			       0,		// Payload size
			       buf);		// Buffer

	if(my_ret == -1) ERROR("apsr_ethernet failed!");

        my_ret = apsr_pppoe(1,				// Version
			    1,				// Type
			    APSR_PPPOE_CODE_SESS,	// Code
			    0x127f,			// Session ID
			    10,				// Length
			    NULL,			// PPPoE payload
			    0,				// PPPoE payload length
			    buf + APSR_ETH_H);		// Packet buffer

	if(my_ret == -1) ERROR("apsr_pppoe failed!");

	my_ret = apsr_ppp(0x21,				// Protocol
			  NULL,                         // PPP payload
		          0,                            // PPP payload length
			  buf + APSR_ETH_H + APSR_PPPoE_H); // Packet buffer

	if(my_ret == -1) ERROR("apsr_ppp failed!");
	
	/* Send it */
	written = apsr_write_link_layer(apsr_ll, device, buf,
					APSR_ETH_H + APSR_PPPoE_H + APSR_PPP_H, 1);

	apsr_output("Wrote %i bytes!\n", written);
	
	/* Close link layer */
	apsr_close_link_layer(apsr_ll);
	
	return(RET_SUCCESS);
}

