#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"htcp"
#define MODULE_DESCRIPTION	"Example Hyper Text Caching Protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	4827	
#define MODULE_IDENT_ID2	4827

#define MODULE_FILTER		"(tcp or udp) and port 4827"

struct mod_htcp_header {
	unsigned int message_length;
	unsigned char major_version;
	unsigned char minor_version;
	unsigned int data_length:16,
		     opcode:4,
		     response:4,
                     reserved:6,
                     rd_mo:1,
	             rr:1;
	unsigned int trans_id;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_htcp_header *htcp_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    htcp_header = (struct mod_htcp_header *)buf;
	    apsr_output("- HTCP Header -\n");
	    apsr_output("  Message Length: %u, Major Version: %u, Minor Version: %u\n", 
			    apsr_ntohl(htcp_header->message_length),
			    htcp_header->major_version,
			    htcp_header->minor_version);
	    apsr_output("  Data Length: %u, Opcode: %u, Response: %u\n",
			    apsr_ntohs(htcp_header->data_length),
			    htcp_header->opcode,
			    htcp_header->response);
	    apsr_output("  Reserved: %u, RD/MO: %u, RR: %u\n",
			    htcp_header->reserved,
			    htcp_header->rd_mo,
			    htcp_header->rr);
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
