#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.06"
#define MODULE_NAME		"icq"
#define MODULE_DESCRIPTION	"Example ICQ protocol sniffer. Currently know v2/v3 and v4 headers, devel v5/v8."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_PN2	APSR_PROTO_TCP
#define MODULE_IDENT_ID1	4000
#define MODULE_IDENT_ID2	4000
#define MODULE_IDENT_ID3	5190
#define MODULE_IDENT_ID4	5190	

#define MODULE_FILTER		"(udp or tcp) and (port 4000 or port 5190)"

/* There are different server->client and client->server commands, but
   they always mean the same, so I decided to make no difference */
#define APSR_ICQ_COMMAND_HEADER1	0x0003 /* SRV_HEADER (v3) */
#define APSR_ICQ_COMMAND_HEADER2	0x0005 /* CMD_HEADER/SRV_HEADER (v5) */
#define APSR_ICQ_COMMAND_ACK		0x000A /* CMD_ACK / SRV_ACK (v5) */
#define APSR_ICQ_COMMAND_GO_AWAY	0x0028 /* SRV_GO_AWAY (v5) */
#define APSR_ICQ_COMMAND_NEW_UIN	0x0046 /* SRV_NEW_UIN (v3) */
#define APSR_ICQ_COMMAND_LOGIN_REPLY	0x005A /* SRV_LOGIN_REPLY (v5) */
#define APSR_ICQ_COMMAND_BAD_PASS	0x0064 /* SRV_BAD_PASS (v3) */
#define APSR_ICQ_COMMAND_USER_ONLINE	0x006E /* SRV_USER_ONLINE (v5) */
#define APSR_ICQ_COMMAND_USER_OFFLIN	0x0078 /* SRV_USER_OFFLINE (v5) */
#define APSR_ICQ_COMMAND_AWAY_REPLY	0x00E6 /* SRV_X2 (v5) */
#define APSR_ICQ_COMMAND_SEND_MESS	0x010E /* CMD_SEND_MESSAGE (v5) */
#define APSR_ICQ_COMMAND_INFO_ID	0x0118 /* SRV_INFO_REPLY (v5) */
#define APSR_ICQ_COMMAND_UPDATE_SUCC	0x01E0 /* SRV_UPDATE_SUCCESS (v2) */
#define APSR_ICQ_COMMAND_UPDATE_FAIL	0x01EA /* SRV_UPDATE_FAIL (v2) */
#define APSR_ICQ_COMMAND_LOGIN		0x03E8 /* CMD_LOGIN (v5) */
#define APSR_ICQ_COMMAND_REG_NEWUSER	0x03FC /* CMD_REG_NEW_USER (v5) */
#define APSR_ICQ_COMMAND_PING		0x0409
#define APSR_ICQ_COMMAND_SEARCH_UIN	0x041A /* CMD_SEARCH_UIN (v2) */
#define APSR_ICQ_COMMAND_DISCONNECT	0x0438 /* CMD_SEND_TEXT_CODE (v5) */
#define APSR_ICQ_COMMAND_ACK_MESS	0x0442 /* CMD_ACK_MESSAGES (v5) */
#define APSR_ICQ_COMMAND_MESSAGE	0x0456 /* CMD_MSG_TO_NEW_USER (v5) */
#define APSR_ICQ_COMMAND_INFO		0x0460 /* CMD_INFO_REQ (v5) */
#define APSR_ICQ_COMMAND_INFO2		0x046A /* CMD_EXT_INFO_REQ (v5) */
#define APSR_ICQ_COMMAND_STATUS		0x04D8 /* CMD_STATUS_CHANGE (v5) */

#define APSR_ICQV8_FLAP_ID		0x2A   /* FLAP Header */

#define APSR_ICQV8_CHANNEL_SNAC		0x02   /* SNAC Header */

/* ICQv8 Channels */
static const char *mod_icqv8_channel[]= {
	" (Unkown)",
	" (Login)",
	" (SNAC)",
	" (Error)",
	" (Disconnect)",
	" (Ping)",
};

/* ICQv2/v3/v4 header */
struct mod_icq_header {
	unsigned char version[1];
	union {
		struct {
			unsigned short command;
			unsigned short seq_num;
			union {
				struct {
					unsigned int uin;
				} c_to_s; /* Client to Server communication */
			} cs;
		} v2;	
		struct {
			unsigned short command;
			unsigned short seq_code1;
			unsigned short seq_code2;
			unsigned int uin;
			unsigned int sum;
		} v3;
                struct {
                        unsigned short random;
			unsigned short zeros;
			unsigned short command;
			unsigned short seq_code1;
			unsigned short seq_code2;
			unsigned int uin;
			unsigned int sum;
 		} v4;								
	} ext;
};

/* ICQv8 FLAP header */
struct mod_icq_flap_header {
	unsigned char id;
	unsigned char channel;
	unsigned short seq;
	unsigned short length;
};

/* ICQv8 SNAC header */
struct mod_icq_snac_header {
	unsigned short family;
	unsigned short command;
 	unsigned short flags; 
	unsigned int reference;
};

/* ICQv8 TLV header */
/*struct mod_icq_tlv_header { 
	unsigned short type;
	unsigned short length;
};*/

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER;
  	module_info.family      = MODULE_IDENT_FAMILY;
    	module_info.proto1      = MODULE_IDENT_PN1;
      	module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
	module_info.id2         = MODULE_IDENT_ID2;
	module_info.id3         = MODULE_IDENT_ID3;
	module_info.id4         = MODULE_IDENT_ID4;
	      
	return(module_info);
}

extern char verbose;

/* Prototype */
void print_icq_command(unsigned short);

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_icq_header *icq_header;
  struct mod_icq_flap_header *icq_flap_header;
  struct mod_icq_snac_header *icq_snac_header;
  unsigned short command;

  /* UDP ICQv2/v3/v4/v5 */
  if(MODULE_IDENT_FAMILY == family && MODULE_IDENT_PN1 == protocol_number && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
    icq_header = (struct mod_icq_header *)buf;
    apsr_output("- ICQ Header -\n");
    apsr_output("  Major version: %d, Minor version: %d\n", icq_header->version[0], icq_header->version[1]);
    if(icq_header->version[0] == 2) {
      command = apsr_ntohs(icq_header->ext.v2.command);
      apsr_output("  Command: %u", command);
      if(verbose)
        print_icq_command(command);
      apsr_output("\n  Sequence Number: %u\n", apsr_ntohs(icq_header->ext.v2.seq_num));
    }
    else if(icq_header->version[0] == 3) {
      command = apsr_ntohs(icq_header->ext.v3.command);
      apsr_output("  Command: %u", command);
      if(verbose)
        print_icq_command(command);
      apsr_output("\n  First Seq. Code: %u\n", apsr_ntohs(icq_header->ext.v3.seq_code1));
      apsr_output("  Second Seq. Code: %u\n", apsr_ntohs(icq_header->ext.v3.seq_code2));
      apsr_output("  UIN: %d\n", apsr_ntohl(icq_header->ext.v3.uin));
      apsr_output("  Checksum: 0x%04x\n", apsr_ntohl(icq_header->ext.v3.sum));
    }
    else if(icq_header->version[0] == 4) {
      apsr_output("  Random number: %u\n", apsr_ntohs(icq_header->ext.v4.random));
    }
    else 
      apsr_output("  <Not implemented @ the moment!>\n");
  }
  /* TCP ICQv7/v8/v9 */
  else if(MODULE_IDENT_FAMILY == family && MODULE_IDENT_PN2 == protocol_number && (MODULE_IDENT_ID3 == id1 || MODULE_IDENT_ID4 == id2)) {
    icq_flap_header = (struct mod_icq_flap_header *)buf;
    if(icq_flap_header->id == APSR_ICQV8_FLAP_ID) {
      apsr_output("- ICQ Flap Header -\n");
      apsr_output("  ID: 0x%02x, Channel: %u", 
       icq_flap_header->id, icq_flap_header->channel);
      if(verbose)
        apsr_output("%s",mod_icqv8_channel[icq_flap_header->channel]);
      apsr_output("\n  Sequence: %d, Length: %d\n",
        apsr_ntohs(icq_flap_header->seq),
        apsr_ntohs(icq_flap_header->length));
      if(icq_flap_header->channel == APSR_ICQV8_CHANNEL_SNAC) {
        icq_snac_header = (struct mod_icq_snac_header *)buf+6;	
	apsr_output("- ICQ SNAC Header -\n");
	apsr_output("  Family: 0x%04x, Command: 0x%04x\n",
	  apsr_ntohs(icq_snac_header->family),
	  apsr_ntohs(icq_snac_header->command));
      }
    }
    else {
      apsr_output("  <Not implemented @ the moment!>\n");
    }
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}

void print_icq_command(unsigned short command) {
	switch(command) {
		case APSR_ICQ_COMMAND_HEADER1:
	          apsr_output(" (Srv header)");
		  break;
		case APSR_ICQ_COMMAND_HEADER2:
	          apsr_output(" (Header)");
		  break;
		case APSR_ICQ_COMMAND_ACK:
	          apsr_output(" (Ack)");
		  break;
		case APSR_ICQ_COMMAND_GO_AWAY:
		  apsr_output(" (Go Away)");
		  break;
		case APSR_ICQ_COMMAND_NEW_UIN:
		  apsr_output(" (New UIN)");
		  break;
		case APSR_ICQ_COMMAND_LOGIN_REPLY:
		  apsr_output(" (Login reply)");
		  break;
		case APSR_ICQ_COMMAND_BAD_PASS:
		  apsr_output(" (Bad password)");
		  break;
		case APSR_ICQ_COMMAND_USER_OFFLIN:
		  apsr_output(" (User offline)");
		  break;
		case APSR_ICQ_COMMAND_USER_ONLINE:
		  apsr_output(" (User online)");
		  break;
	 	case APSR_ICQ_COMMAND_SEND_MESS:
		  apsr_output(" (Send message)");
		  break;  
		case APSR_ICQ_COMMAND_UPDATE_SUCC:
		  apsr_output(" (Update success)");
		  break;
		case APSR_ICQ_COMMAND_UPDATE_FAIL:
		  apsr_output(" (Update fail)");
		  break;
		case APSR_ICQ_COMMAND_LOGIN:
		  apsr_output(" (Login)");
		  break;
		case APSR_ICQ_COMMAND_REG_NEWUSER:
		  apsr_output(" (Register new user)");
		  break;
		case APSR_ICQ_COMMAND_PING:
		  apsr_output(" (Ping)");
		  break;
		case APSR_ICQ_COMMAND_SEARCH_UIN:
		  apsr_output(" (Search UIN)");
		  break;
		case APSR_ICQ_COMMAND_DISCONNECT:
		  apsr_output(" (Disconnect)");
		  break;
		case APSR_ICQ_COMMAND_ACK_MESS:
		  apsr_output(" (Ack message)");
		  break;
		case APSR_ICQ_COMMAND_MESSAGE:
		  apsr_output(" (Message)");
		  break;
		case APSR_ICQ_COMMAND_INFO:
		  apsr_output(" (Information)");
		  break;
		case APSR_ICQ_COMMAND_INFO2:
		  apsr_output(" (Information_2)");
		  break;
		case APSR_ICQ_COMMAND_STATUS:
		  apsr_output(" (Status)");
		  break;
		case APSR_ICQ_COMMAND_INFO_ID:
		  apsr_output(" (Info ID)");
		  break;
		case APSR_ICQ_COMMAND_AWAY_REPLY:
		  apsr_output(" (Away reply)");
		  break;
		default:
		  apsr_output(" (Unknown)");
		  break;
	}
}
