#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"l2tp"
#define MODULE_DESCRIPTION	"Example level 2 tunneling protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	1701
#define MODULE_IDENT_ID2	1701

#define MODULE_FILTER		"udp and port 1701"

struct mod_l2tp_header {
	unsigned char bits;
	unsigned char version;
	unsigned short length;
	unsigned short tunnel_id;
	unsigned short call_id;
	unsigned short ns;
	unsigned short nr;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_l2tp_header *l2tp_header;
  unsigned char type;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    l2tp_header = (struct mod_l2tp_header *)buf;

	    type = l2tp_header->bits && 0x7F;
	    
	    if(type == 0) {
		    apsr_output("- L2TP Data Header -\n");
	    }
	    else {
		    apsr_output("- L2TP Control Header -\n");

		    apsr_output("  Bits: 0x%02x, Version: %u\n", 
				    l2tp_header->bits,
				    l2tp_header->version);
		    apsr_output("  Length: %u, Tunnel ID: %u, Call ID: %u\n",
				    apsr_ntohs(l2tp_header->length),
				    apsr_ntohs(l2tp_header->tunnel_id),
				    apsr_ntohs(l2tp_header->call_id));
		    apsr_output("  Next Sent: %u, Next Received: %u\n",
				    apsr_ntohs(l2tp_header->ns),
			            apsr_ntohs(l2tp_header->nr));	    
  
	    }
   }
	    
  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
