#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"rap"
#define MODULE_DESCRIPTION	"Example Route Access Protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_TCP
#define MODULE_IDENT_PN2	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	38	
#define MODULE_IDENT_ID2	38

#define MODULE_FILTER		"(tcp or udp) and port 38"

static const char *rap_commands[]= {
	" (No operation)",
	" (Poll)",
	" (Error)",
};

struct mod_rap_header {
	unsigned short version;
	unsigned short command;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = MODULE_IDENT_PN2;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_rap_header *rap_header;
  unsigned short command;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    rap_header = (struct mod_rap_header *)buf;
	    command = apsr_ntohs(rap_header->command);
	    
	    apsr_output("- RAP Header -\n");
	    apsr_output("  Version: %u, Command: %u", 
			    apsr_ntohs(rap_header->version),
			    apsr_ntohs(rap_header->command));
            if(verbose)
		    apsr_output("%s", rap_commands[command]);
	    apsr_output("\n");
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
