#include <apsrlib/apsrlib.h>

#define MODULE_VERSION          "0.02"
#define MODULE_NAME             "tftp"
#define MODULE_DESCRIPTION      "simple tftp sniffer"
#define MODULE_IDENT_FAMILY     AF_INET
#define MODULE_IDENT_PN1        APSR_PROTO_UDP
#define MODULE_IDENT_ID1        69
#define MODULE_IDENT_ID2        69

#define MODULE_FILTER           "udp and port 69"


#define APSR_TFTP_RRQ     1		/* read request */
#define APSR_TFTP_WRQ     2		/* write request */
#define APSR_TFTP_DATA    3		/* data packet */
#define APSR_TFTP_ACK     4		/* acknowledgement */
#define APSR_TFTP_ERROR   5		/* error code */

static const char *mod_tftp_opcode[] = {
	" (Unknown)",
	" (RRQ)",
	" (WRQ)",
	" (DATA)",
	" (ACK)",
	" (ERROR)",
};

struct mod_tftp_header {
  unsigned short int th_opcode; 
  union {
    unsigned short int tu_block;       
    unsigned short int tu_code;                
    unsigned char  tu_stuff[1];            
  } th_u;
  unsigned char th_data[1];                     
}; 



struct apsr_aprecv_module_info module_info (void) {
  struct apsr_aprecv_module_info module_info;

  module_info.version     = MODULE_VERSION;
  module_info.name        = MODULE_NAME;
  module_info.desc        = MODULE_DESCRIPTION;
  module_info.bpf_filter  = MODULE_FILTER;
  module_info.family      = MODULE_IDENT_FAMILY;
  module_info.proto1      = MODULE_IDENT_PN1;
  module_info.proto2      = 0;
  module_info.id1         = MODULE_IDENT_ID1;
  module_info.id2         = MODULE_IDENT_ID2;
  module_info.id3         = 0;
  module_info.id4         = 0;

  return(module_info);
}

extern char verbose;
extern unsigned short module_length;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_tftp_header *tftp_header;
  unsigned char *buffer;
  unsigned short int opcode, i=0, x=0, k=0;

  tftp_header = (struct mod_tftp_header *)buf;
  opcode = apsr_ntohs(tftp_header->th_opcode);

  apsr_output("- TFTP Header -\n");
  apsr_output("  Opcode: %d", opcode);

  if(verbose)
	  apsr_output("%s", mod_tftp_opcode[opcode]);

  switch(opcode) {
  case APSR_TFTP_RRQ:
  case APSR_TFTP_WRQ:
    buffer = (unsigned char *)(buf+2);

    apsr_output("\n  Filename: \"");
    for(i = 0; i < module_length; i++) { 
	    if(buffer[i] == 0) {
		    if(x == 1)
			    i = module_length; 
	            else {
			    x += 1;
			    apsr_output("\"\n  Mode: \"");
		    }
            }
	    else 
		    apsr_output("%c", buffer[i]);
    }
    apsr_output("\"\n");

    break;
  case APSR_TFTP_ACK:
  case APSR_TFTP_DATA:
    apsr_output("Block: %d", tftp_header->th_u.tu_block);
    if(opcode == 3) {
      buffer=(unsigned char *)(buf+4);
	for(i = 0; i < module_length; i++) {
	  apsr_output("Data: %s", buffer[i]);
	}
    }  
    break;
  case APSR_TFTP_ERROR:
    apsr_output("Errorcode: %d", ntohs(tftp_header->th_u.tu_block));
    buffer=(unsigned char *)(buf+4);
    for(k = 0; k < module_length; k++) {
      if(buffer[k] == 0)
	i = module_length;
      else {
	apsr_output("\"\n  Errormessage: \"%c\"", buffer[k]);
      }
    }
    break;
  default:
    break;
  }
  
  return(RET_SUCCESS);
}
