#include <apsrlib/apsrlib.h>

#define MODULE_VERSION		"0.01"
#define MODULE_NAME		"WCCP"
#define MODULE_DESCRIPTION	"Example Web Cache Coordination Protocol sniffer."
#define MODULE_IDENT_FAMILY	AF_INET
#define MODULE_IDENT_PN1	APSR_PROTO_UDP
#define MODULE_IDENT_ID1	2048
#define MODULE_IDENT_ID2	2048

#define MODULE_FILTER		"udp and port 2048"

#define WCCP1_HERE_I_AM		7
#define WCCP1_I_SEE_YOU		8
#define WCCP1_ASSIGN_BUCKET	9
#define WCCP2_HERE_I_AM         10
#define WCCP2_I_SEE_YOU         11
#define WCCP2_REDIRECT_ASSIGN   12
#define WCCP2_REMOVAL_QUERY     13

/* draft-ietf-wrec-web-pro-00.txtdraft-ietf-wrec-web-pro-00.txt - v1
 * draft-wilson-wrec-wccp-v2-00.txt				- v2
 * draft-wilson-wrec-wccp-v2-01.txt 				- v2 */

struct mod_wccp_header {
	unsigned int type;
};

struct apsr_aprecv_module_info module_info (void) {
	struct apsr_aprecv_module_info module_info;

	module_info.version	= MODULE_VERSION;
	module_info.name	= MODULE_NAME;
	module_info.desc	= MODULE_DESCRIPTION;
	module_info.bpf_filter	= MODULE_FILTER; 
        module_info.family      = MODULE_IDENT_FAMILY;
        module_info.proto1      = MODULE_IDENT_PN1;
        module_info.proto2      = 0;
        module_info.id1         = MODULE_IDENT_ID1;
        module_info.id2         = MODULE_IDENT_ID2;
        module_info.id3         = 0;
        module_info.id4         = 0;
							
	return(module_info);
}

extern char verbose;

int module_init (unsigned char *buf, int family, unsigned short protocol_number,
                 unsigned short id1, unsigned short id2)
{
  struct mod_wccp_header *wccp_header;
	
  if(MODULE_IDENT_FAMILY == family && (MODULE_IDENT_ID1 == id1 || MODULE_IDENT_ID2 == id2)) {
	    wccp_header = (struct mod_wccp_header *)buf;
	    apsr_output("- WCCP Header -\n");
	    apsr_output("  Type: %u", apsr_ntohl(wccp_header->type));
	    if(verbose) {
		    switch(apsr_ntohl(wccp_header->type)) {
			    case WCCP1_HERE_I_AM:
			      apsr_output(" (v1/Here I am)");
			      break;
			    case WCCP1_I_SEE_YOU:
			      apsr_output(" (v1/I see you)");
			      break;
			    case WCCP1_ASSIGN_BUCKET:
			      apsr_output(" (v1/Assign bucket)");
			      break;
			    case WCCP2_HERE_I_AM:
			      apsr_output(" (v2/Here I am)");
			      break;
			    case WCCP2_I_SEE_YOU:
			      apsr_output(" (v2/I see you)");
			      break;
			    case WCCP2_REDIRECT_ASSIGN:
			      apsr_output(" (v2/Redirect assign)");
			      break;
			    case WCCP2_REMOVAL_QUERY:
			      apsr_output(" (v2/Removal query)");
			      break;
			    default:
			      apsr_output(" (Unknown)");
			      break;
	            }				      
	    }
	    apsr_output("\n");
  }

  #ifdef MODULE_DEBUG
  apsr_output("Family: %i\nPN: %i\n1: %i\n2: %i\n",
     family, protocol_number, id1, id2);
  #endif
  
  return(RET_SUCCESS);
}
